package de.duehl.basics.system.starter;

/*
 * Copyright 2016 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import de.duehl.basics.datetime.Timestamp;
import de.duehl.basics.io.FineFileWriter;
import de.duehl.basics.logging.Logger;
import de.duehl.basics.logic.ErrorHandler;

/**
 * Diese Klasse ruft eine cmd-Datei mit einem Programm auf und wartet auf die
 * Ausführung.
 * Sollte also nur aus einem Worker-Thread gestartet werden!
 *
 * @version 1.01     2014-01-09
 * @author Christian Dühl
 */

public class CmdStarter extends Starter {

    /**
     * Konstruktor.
     *
     * @param error
     *            Objekt, das die Fehlerbehandlung durchführt.
     */
    public CmdStarter(final ErrorHandler error) {
        this(null, error);
    }

    /**
     * Konstruktor.
     *
     * @param logger
     *            Der Logger für den Ablauf dieser Session.
     * @param error
     *            Objekt, das die Fehlerbehandlung durchführt.
     */
    public CmdStarter(final Logger logger, final ErrorHandler error) {
        super(logger, error);
    }

    /**
     * Führt eine Cmd-Datei aus und wartet, bis es fertig ist.
     *
     * @param program
     *            Auszuführendes Cmd-Programm.
     * @param log
     *            Logfile, das erzeugt wird.
     * @param errorLog
     *            Extrakt der Fehler aus dem Logfile, wird ebenfalls erzeugt.
     * @return Gestarteter Aufruf.
     */
    public String runAndWait(final String program, final String log,
            final String errorLog) {
        log("Start, program = " + program
                + ", log = " + log + ", errorLog = " + errorLog);

        final String[] params = {
                "cmd",
                "/c",
                program
        };
        ProcessBuilder processBuilder = new ProcessBuilder(params);
        Process process = null;
        try {
            process = processBuilder.start();
        }
        catch (IOException exception) {
            error.error("Es trat ein Fehler bei der Ausführung des Cmd"
                    + "-Prozesses auf.", exception);
            return null;
        }

        /* Auf Beendigung warten. */
        try {
            process.waitFor();
        }
        catch (InterruptedException e) {
            // nichts, das kann nicht unterbrochen werden.
        }


        BufferedReader in = new BufferedReader(new InputStreamReader(
                process.getInputStream()));
        FineFileWriter outWriter = new FineFileWriter(log);
        try {
            for (String line; (line = in.readLine()) != null;) {
                outWriter.writeln(line);
            }
        }
        catch (IOException exception) {
            error.error("Es trat ein Fehler bei der Ausführung des Cmd"
                    + "-Prozesses beim Ermitteln des Outputs auf.", exception);
        }
        outWriter.close();

        BufferedReader err = new BufferedReader(new InputStreamReader(
                process.getErrorStream()));
        FineFileWriter errWriter = new FineFileWriter(errorLog);
        try {
            for (String line; (line = err.readLine()) != null;) {
                errWriter.writeln(line);
            }
        }
        catch (IOException exception) {
            error.error("Es trat ein Fehler bei der Ausführung des Cmd"
                    + "-Prozesses beim Ermitteln des Outputs auf.", exception);
        }
        errWriter.close();


        log("Ende, program = " + program
                + ", log = " + log + ", errorLog = " + errorLog);

        /* Aufgerufenen Befehl als String zurückliefern: */
        return paramsToString(params);
    }

    /**
     * Führt eine Cmd-Datei aus und wartet, bis es fertig ist.
     *
     * @param program
     *            Auszuführendes Cmd-Programm.
     * @param log
     *            Logfile, das erzeugt wird.
     * @param errorLog
     *            Extrakt der Fehler aus dem Logfile, wird ebenfalls erzeugt.
     * @return Gestarteter Aufruf.
     */
    public String runAndForget(final String program) {
        log("Start, program = " + program);

        final String[] params = {
                "cmd",
                "/c",
                program
        };
        ProcessBuilder processBuilder = new ProcessBuilder(params);
        try {
            processBuilder.start();
        }
        catch (IOException exception) {
            error.error("Es trat ein Fehler bei der Ausführung des Cmd"
                    + "-Prozesses auf.", exception);
            return null;
        }

        log("Ende, program = " + program);
        return paramsToString(params);
    }

    /**
     * Startet das Ausführen einer Cmd-Datei.
     *
     * @param program
     *            Auszuführendes Cmd-Programm.
     * @return Gestarteter Aufruf.
     */
    public Process startRun(final String program) {
        log("Start, program = " + program);

        final String[] params = {
                "cmd",
                "/c",
                program
        };
        ProcessBuilder processBuilder = new ProcessBuilder(params);
        Process process = null;
        try {
            process = processBuilder.start();
        }
        catch (IOException exception) {
            error.error("Es trat ein Fehler bei der Ausführung des Cmd"
                    + "-Prozesses auf.", exception);
            return null;
        }

        log("Ende, program = " + program);
        return process;
    }

    /**
     * Startet das Ausführen einer Cmd-Datei.
     *
     * @param params
     *            Teile des zu startenden Aufrufs.
     * @return Gestarteter Aufruf.
     */
    public Process startRunWithoutCmd(final String ... params) {
        log("Start, params = " + params);

        ProcessBuilder processBuilder = new ProcessBuilder(params);
        Process process = null;
        try {
            process = processBuilder.start();
        }
        catch (IOException exception) {
            error.error("Es trat ein Fehler bei der Ausführung des Cmd"
                    + "-Prozesses auf.", exception);
            return null;
        }

        log("Ende, params = " + params);
        return process;
    }

    /**
     * Wartet auf die Beendigung eines gestarteten Processes.
     *
     * @param process
     *            Gestarteter Prozess, auf den gewartet werden soll.
     * @param log
     *            Logfile, das erzeugt wird.
     * @param errorLog
     *            Extrakt der Fehler aus dem Logfile, wird ebenfalls erzeugt.
     */
    public void waitForProcess(final Process process, final String log,
            final String errorLog) {
        log("Start, process = " + process + ", log = " + log + ", errorLog = "
                + errorLog);

        /* Auf Beendigung warten: */
        waitForProcess(process);

        /*
         * Falls der Process gewaltsam mit destroy() beendet wurde, ermitteln
         * wir keine Log- und Fehlerdateien:
         */
        if (0 != process.exitValue()) {
            return;
        }


        BufferedReader in = new BufferedReader(new InputStreamReader(
                process.getInputStream()));
        FineFileWriter outWriter = new FineFileWriter(log);
        try {
            for (String line; (line = in.readLine()) != null;) {
                outWriter.writeln(line);
            }
        }
        catch (IOException exception) {
            error.error("Es trat ein Fehler bei der Ausführung des Cmd"
                    + "-Prozesses beim Ermitteln des Outputs auf.", exception);
        }
        outWriter.close();
        System.out.println(Timestamp.actualTime()
                + " CmdStarter#waitForProcess der "
                + "Prozess '" + process + "' STDOUT ermittelt."); // TODO REMOVE

        BufferedReader err = new BufferedReader(new InputStreamReader(
                process.getErrorStream()));
        FineFileWriter errWriter = new FineFileWriter(errorLog);
        try {
            for (String line; (line = err.readLine()) != null;) {
                errWriter.writeln(line);
            }
        }
        catch (IOException exception) {
            error.error("Es trat ein Fehler bei der Ausführung des Cmd"
                    + "-Prozesses beim Ermitteln des Outputs auf.", exception);
        }
        errWriter.close();
        System.out.println(Timestamp.actualTime()
                + " CmdStarter#waitForProcess der "
                + "Prozess '" + process + "' STDERR ermittelt."); // TODO REMOVE

        log("Ende, process = " + process + ", log = " + log + ", errorLog = "
                + errorLog);
    }

    /**
     * Wartet auf die Beendigung eines gestarteten Processes, ohne dessen
     * Ausgaben nach STDOUT und STDERR zu ermitteln.
     *
     * @param process
     *            Gestarteter Prozess, auf den gewartet werden soll.
     */
    public void waitForProcess(final Process process) {
        log("Start, process = " + process);

        /* Auf Beendigung warten. */
        try {
            process.waitFor();
        }
        catch (InterruptedException e) {
            // nichts, das kann nicht unterbrochen werden.
        }
        System.out.println(Timestamp.actualTime()
                + " CmdStarter#waitForProcess der "
                + "Prozess '" + process + "' wurde beendet."); // TODO REMOVE

        log("Ende, process = " + process);
    }

}
