package de.duehl.basics.text;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Diese Klasse zerlegt Text so, dass er in eine bestimmte Zeilenbreite passt.
 *
 * @version 1.01     2018-03-27
 * @author Christian Dühl
 */

public class TextBreaker {

    /** Systemunabhängiger Zeilenumbruch. */
    static final String LINE_BREAK = System.getProperty("line.separator");

    /** Gewünschte Zeilenlänge. */
    private final int lineLength;

    /** Anzahl einzufügender Leerzeichen am Anfang der Zeile. */
    private final int numberOfFrontSpaces;

    /**
     * Konstruktor.
     *
     * @param lineLength
     *            Gewünschte Zeilenlänge.
     */
    public TextBreaker(int lineLength) {
        this(lineLength, 0);
    }

    /**
     * Konstruktor.
     *
     * @param lineLength
     *            Gewünschte Zeilenlänge.
     * @param numberOfFrontSpaces
     *            Anzahl einzufügender Leerzeichen am Anfang der Zeile.
     */
    public TextBreaker(int lineLength, int numberOfFrontSpaces) {
        if (numberOfFrontSpaces >= lineLength) {
            throw new IllegalArgumentException("Zu wenig Platz für den umzubrechenden Text!");
        }

        this.lineLength = lineLength - numberOfFrontSpaces;
        this.numberOfFrontSpaces = numberOfFrontSpaces;
    }

    /**
     * Versieht einen langen Text an Leerzeichen mit Zeilenumbrüchen, so dass die Zeilen höchstens
     * lineLength Zeichen lang sind, - falls ausreichend Leerzeichen dafür vorhanden sind.
     *
     * Hierbei wird der Text zuvor an schon vorhandenen Umbrüchen umgebrochen.
     *
     * @param text
     *            Gegebener Text.
     * @return Umgebrochener Text.
     */
    public String addLineBreaks(String text) {
        List<String> brokenParts = new ArrayList<>();

        for (String part : breakTextBetweenExistingLineBreaks(text)) {
            brokenParts.addAll(breakAtSpaces(part));
        }

        return generateOutputString(brokenParts);
    }

    private List<String> breakTextBetweenExistingLineBreaks(String text) {
        return Arrays.asList(text.split("(" + LINE_BREAK + "|\r\n|\n)", -1));
    }

    private String generateOutputString(List<String> brokenParts) {
        StringBuilder builder = new StringBuilder();

        boolean first = true;
        for (String brokenPart : brokenParts) {
            if (first) {
                first = false;
            }
            else {
                builder.append(LINE_BREAK);
                builder.append(Text.multipleString(' ', numberOfFrontSpaces));
            }
            builder.append(brokenPart);
        }

        return builder.toString();
    }

    private List<String> breakAtSpaces(String part) {
        List<String> brokenParts = new ArrayList<>();

        if (textIsSmallEnough(part) || textHasNoSpaces(part)) {
            brokenParts.add(part);
        }
        else {
            brokenParts.addAll(breakAtBestSpace(part));
        }

        return brokenParts;
    }

    private List<String> breakAtBestSpace(String part) {
        List<String> brokenParts = new ArrayList<>();

        int devidingIndex;

        int firstSpaceIndex = part.indexOf(" ");

        if (textIsSmallEnoughAtSpaceIndex(firstSpaceIndex)) {
            int lastGoodSpaceIndex = firstSpaceIndex;
            int aSpaceIndex = part.indexOf(" ", lastGoodSpaceIndex + 1);
            while (aSpaceIndex > -1 && textIsSmallEnoughAtSpaceIndex(aSpaceIndex)) {
                lastGoodSpaceIndex = aSpaceIndex;
                aSpaceIndex = part.indexOf(" ", lastGoodSpaceIndex + 1);
            }
            devidingIndex = lastGoodSpaceIndex;
        }
        else {
            devidingIndex = firstSpaceIndex;
        }

        String partBeforeSpace = part.substring(0, devidingIndex);
        String partAfterSpace = part.substring(devidingIndex + 1);
        brokenParts.add(partBeforeSpace);
        brokenParts.addAll(breakAtSpaces(partAfterSpace));

        return brokenParts;
    }

    private boolean textIsSmallEnough(String part) {
        return part.length() <= lineLength;
    }

    private boolean textIsSmallEnoughAtSpaceIndex(int spaceIndex) {
        return spaceIndex <= lineLength;
        /*
         * Nicht
         *     spaceIndex -1 <= lineLength
         * weil das Leerzeichen ja nicht mitzählt!
         */
    }

    private boolean textHasNoSpaces(String text) {
        return !text.contains(" ");
    }

}
