package de.duehl.math.graph.ged.graph;

import java.util.ArrayList;
import java.util.List;

import de.duehl.basics.text.html.generation.HtmlBuilder;
import de.duehl.math.geometry.Line;
import de.duehl.math.graph.ged.graph.edge.Edge;
import de.duehl.math.graph.ged.graph.edge.Edges;
import de.duehl.math.graph.ged.graph.io.Ged1FormatLoader;
import de.duehl.math.graph.ged.graph.io.Ged1FormatSaver;
import de.duehl.math.graph.ged.graph.io.Ged2FormatLoader;
import de.duehl.math.graph.ged.graph.io.Ged2FormatSaver;
import de.duehl.math.graph.ged.graph.io.ged3.Ged3FormatLoader;
import de.duehl.math.graph.ged.graph.io.ged3.Ged3FormatSaver;
import de.duehl.math.graph.ged.graph.io.ged4.Ged4FormatLoader;
import de.duehl.math.graph.ged.graph.io.ged4.Ged4FormatSaver;
import de.duehl.math.graph.ged.graph.subGraph.EdgeSubGraphCreator;
import de.duehl.math.graph.ged.graph.subGraph.VertexWithItsNeigboursSubGraphCreator;
import de.duehl.math.graph.ged.graph.vertex.Vertex;
import de.duehl.math.graph.ged.graph.vertex.Vertices;
import de.duehl.math.graph.ged.pixel.PixelPoint;

/**
 * Diese Klasse stellt einen Graphen dar.
 *
 * @version 1.01     2019-06-19
 * @author Christian Dühl
 */

public class Graph {

    /** Eckenmenge des Graphen. */
    private final Vertices vertices;

    /** Kantenmenge des Graphen. */
    private final Edges edges;

    /** Enthält weitere Informationen zum Graphen. */
    private final MetaData meta;

    /** Konstruktor */
    public Graph() {
        vertices = new Vertices();
        edges = new Edges();
        meta = new MetaData();
    }

    /**
     * Erzeugt eine Kopie des übergebenen Graphen.
     *
     * @param that
     *            zu kopierender Graph
     */
    public Graph(Graph that) {
        this();
        synchronized (that) {
            for (Vertex vertex : that.vertices) {
                Vertex copyVertex = new Vertex(vertex);
                vertices.add(copyVertex);
            }
            for (Edge edge : that.edges) {
                Edge copyEdge = new Edge(edge);
                edges.add(copyEdge);
            }
            meta.copyValues(that.meta);
        }
    }

    /**
     * Gibt die Anzahl der Ecken zurück.
     *
     * @return Anzahl der Ecken.
     */
    public int getNumberOfVertices() {
        return vertices.size();
    }

    /**
     * Gibt die Anzahl der Kanten des Graphen zurück.
     *
     * @return Anzahl der Kanten.
     */
    public int getNumberOfEdges() {
        return edges.size();
    }

    /**
     * Überprüft, ob der Graph in sich noch valide ist.
     *
     * @return Wahrheitswert
     */
     boolean validate() { // nicht public
        /*
         * Überprüfen, ob die Indices der Ecken auch alle zu den Indices der
         * Liste passen:
         */
        for (int index = 0; index < vertices.size(); ++index) {
            Vertex vertex = vertices.get(index);
            int vertexIndex = vertex.getIndex();
            if (index != vertexIndex) {
                return false;
            }
        }

        /*
         * Überprüfen ob alle Ecken-Indices der Kanten noch zu existierenden Ecken passen:
         */
        int minVertexIndex = 0;
        int maxVertexIndex = vertices.size() - 1;

        for (Edge edge : edges) {
            if (minVertexIndex > edge.getStartVertexIndex()
                    || minVertexIndex > edge.getTargetVertexIndex()
                    || maxVertexIndex < edge.getStartVertexIndex()
                    || maxVertexIndex < edge.getTargetVertexIndex()) {
                return false;
            }
        }

        return true;
    }

    /** Fügt eine Ecke zum Graphen hinzu. Der Index der Ecke wird automatisch korrekt vergeben. */
    synchronized
    public void addVertex(Vertex vertex) {
        /*
         * Das gleiche Objekt darf nicht mehrfach in der Liste sein, da die
         * Indices sonst bei allen davon gleich wären:
         */
        if (vertices.containsExactObject(vertex)) {
            throw new IllegalArgumentException("Die Ecke " + vertex + " wurde schon hinzugefügt!");
        }

        /* Richtigen Index in der Ecke setzen: */
        int nextIndex = vertices.size();
        vertex.setIndex(nextIndex);

        /* Ecke hinzufügen: */
        vertices.add(vertex);

        /* Ecke im Gitter einrasten lassen, falls gewünscht: */
        if (meta.isSnapInGrid()) {
            snapVertexToGrid(vertex);
        }
    }

    /** Rastet die Ecke im Gitter ein. */
    synchronized
    public void snapVertexToGrid(Vertex vertex) {
        double distance = meta.getGridDistance();
        PixelPoint coord = vertex.getCoordinates();

        /* x */
        double x = coord.getX();
        double xFactor = x / distance;
        double xBefore = Math.floor(xFactor) * distance;
        double xBehind = Math.ceil(xFactor) * distance;
        if (x-xBefore < xBehind-x) {
            x = xBefore;
        }
        else {
            x = xBehind;
        }
        coord.setX(x);

        /* y */
        double y = coord.getY();
        double yFactor = y / distance;
        double yBefore = Math.floor(yFactor) * distance;
        double yBehind = Math.ceil(yFactor) * distance;
        if (y - yBefore < yBehind - y) {
            y = yBefore;
        }
        else {
            y = yBehind;
        }
        coord.setY(y);
    }

    /** Fügt eine Kante zum Graphen hinzu. Die Indices der Start- und Ziel-Ecke müssen passen. */
    synchronized
    public void addEdge(Edge edge) {
        checkVertexIndicesOfNewEdge(edge);
        edges.add(edge);
    }

    private void checkVertexIndicesOfNewEdge(Edge edge) {
        int from = edge.getStartVertexIndex();
        int to = edge.getTargetVertexIndex();
        checkVertexIndicesOfEdge(from, to);
        checkExistingEdge(from, to);
    }

    private void checkVertexIndicesOfEdge(int from, int to) {
        int numberOfVertices = getNumberOfVertices();
        if (from < 0 || from >= numberOfVertices) {
            throw new IllegalArgumentException("Ecke mit Start-Index " + from + " nicht vorhanden.");
        }
        if (to < 0 || to >= numberOfVertices) {
            throw new IllegalArgumentException("Ecke mit End-Index " + to + " nicht vorhanden.");
        }
        if (to == from) {
            throw new IllegalArgumentException("Keine Schleifen!");
        }
    }

    private void checkExistingEdge(int from, int to) {
        if (existsEdge(from, to)) {
            throw new IllegalArgumentException("Es ist bereits eine Kante von fer Ecke mit "
                    + "Start-Index " + from + " zur Ecke mit End-Index " + to + " vorhanden.");
        }
    }

    private void checkVertexIndicex(int vertexIndex) {
        int numberOfVertices = getNumberOfVertices();
        if (vertexIndex < 0 || vertexIndex >= numberOfVertices) {
            throw new IllegalArgumentException("Ecke mit Index " + vertexIndex + " nicht vorhanden.");
        }
    }

    /**
     * Kante zum Graphen hinzufügen.
     *
     * @param startVertexIndex
     *            Index der Start-Ecke (0 bis n-1).
     * @param targetVertexIndex
     *            Index der Ziel-Ecke (0 bis n-1).
     */
    public void addEdge(int startVertexIndex, int targetVertexIndex) {
        addEdge(new Edge(startVertexIndex, targetVertexIndex));
    }

    /**
     * Kante zum Graphen hinzufügen.
     *
     * @param startVertex
     *            Startecke der Kante.
     * @param targetVertex
     *            Endecke der Kante.
     */
    public void addEdge(Vertex startVertex, Vertex targetVertex) {
        int startVertexIndex = startVertex.getIndex();
        int targetVertexIndex = targetVertex.getIndex();
        addEdge(startVertexIndex, targetVertexIndex);
    }

    /**
     * Entfernt eine Kante aus dem Graphen.
     *
     * @param edge
     *            Zu löschende Kante.
     */
    public void removeEdge(Edge edge) {
        removeEdge(edge.getStartVertexIndex(), edge.getTargetVertexIndex());
    }

    /**
     * Entfernt eine Kante aus dem Graphen.
     *
     * @param from
     *            Index der Start-Ecke (0 bis n-1).
     * @param to
     *            Index der Ziel-Ecke (0 bis n-1).
     */
    synchronized
    public void removeEdge(int from, int to) {
        checkVertexIndicesOfEdge(from, to);

        for (Edge edge : edges) {
            if (from == edge.getStartVertexIndex() && to == edge.getTargetVertexIndex()) {
                edges.remove(edge);
                break;
            }
            if (isUndirected()
                    && to == edge.getStartVertexIndex() && from == edge.getTargetVertexIndex()) {
                edges.remove(edge);
                break;

            }
        }
    }

    /**
     * Entfernt eine Kante aus dem Graphen.
     *
     * @param vertexFrom
     *            Startecke der Kante.
     * @param vertexTo
     *            Endecke der Kante.
     */
    public void removeEdge(Vertex vertexFrom, Vertex vertexTo) {
        int from = vertexFrom.getIndex();
        int to = vertexTo.getIndex();
        removeEdge(from, to);
    }

    /** Stringrepräsentation zurückgeben */
    @Override
    public String toString() {
        HtmlBuilder builder = new HtmlBuilder()
                .appendLn("Graph:")
                .increaseIndentationLevel()
                .appendLn("Anzahl Ecken : " + getNumberOfVertices())
                .appendLn("Anzahl Kanten: " + getNumberOfEdges())

                .appendLineBreak()
                .appendLn("Ecken:")
                .increaseIndentationLevel();

        for (Vertex vertex : vertices) {
            builder.appendMultipleLines(vertex.toString());
        }

        builder.appendLineBreak()
                .decreaseIndentationLevel()
                .appendLn("Kanten:")
                .increaseIndentationLevel();

        for (Edge edge : edges) {
            builder.appendMultipleLines(edge.toString());
        }

        builder.appendLineBreak()
                .decreaseIndentationLevel()
                .appendMultipleLines(meta.toString());

        return builder.toString();
    }

    /**
     * Entfernt die Ecke mit dem gegebenen Index.
     *
     * @param index
     *            Index der zu entfernenden Ecke (0 bis n-1).
     */
    synchronized
    public void removeVertex(int index) {
        checkVertexIndicex(index);

        /* Alle Kanten von dieser Ecke weg oder zu dieser Ecke hin entfernen: */
        List<Edge> edgesToDelete = new ArrayList<>();
        for (Edge edge : edges) {
            if (index == edge.getStartVertexIndex() || index == edge.getTargetVertexIndex()) {
                edgesToDelete.add(edge);
            }
        }
        for (Edge edgeToDelete : edgesToDelete) {
            edges.remove(edgeToDelete);
        }

        /* Alle Indices der Ecken mit größerem Index reduzieren: */
        for (int upperIndex = index + 1; upperIndex < vertices.size(); ++upperIndex) {
            Vertex vertex = vertices.get(upperIndex);
            int vertexIndex = vertex.getIndex();
            if (vertexIndex != upperIndex) {
                throw new RuntimeException("Interner Fehler beim Entfernen der " +
                        "Ecke mit Index " + index + " (1).");
            }
            --vertexIndex;
            vertex.setIndex(vertexIndex);
        }

        /* Index der entfernten Ecke auf -1 setzen: */
        vertices.get(index).setIndex(-1);

        /* Ecke selbst entfernen: */
        vertices.remove(index);

        /* Alle Kanten mit größeren Ecken-Indices reduzieren: */
        for (Edge edge : edges) {
            int startVertexIndex = edge.getStartVertexIndex();
            if (startVertexIndex > index) {
                edge.setStartVertexIndex(startVertexIndex - 1);
            }
            int targetVertexIndex = edge.getTargetVertexIndex();
            if (targetVertexIndex > index) {
                edge.setTargetVertexIndex(targetVertexIndex - 1);
            }
        }

        /* Graph überprüfen: */
        if (!validate()) {
            throw new RuntimeException("Interner Fehler beim Entfernen der Ecke mit Index " + index
                    + " (2).");
        }
    }

    /**
     * Entfernt die Ecke.
     *
     * @param vertex
     *            Zu entfernende Ecke.
     */
    public void removeVertex(Vertex vertex) {
        removeVertex(vertex.getIndex());
    }

    /**
     * Verschmilzt die Ecke mit dem gegebenen Index.
     *
     * @param index
     *            Index der zu verschmelzenden Ecke (0 bis n-1).
     */
    synchronized
    public void mergeVertex(int index) {
        /*
         * Neue Kanten einfügen:
         */
        if (isDirected()) {
            insertNewEdgesWhileMergingVertexInDirectedGraph(index);
        }
        else {
            insertNewEdgesWhileMergingVertexInUndirectedGraph(index);
        }

        /* Ecke entfernen: */
        removeVertex(index);
    }

    private void insertNewEdgesWhileMergingVertexInDirectedGraph(int index) {
        /* 1. Nachbarn, von denen Kanten zu dieser Ecke führen, ermitteln: */
        List<Vertex> backwardNeighbours = getBackwardNeighboursAsVertices(index);

        /* 2. Nachbarn, zu denen Kanten von dieser Ecke führen, ermitteln: */
        List<Vertex> forwardNeighbours = getForwardNeighboursAsVertices(index);

        /*
         * 3. Von allen Nachbarn vom Typ 1 zu allen Nachbarn vom Typ 2 Kanten
         * ziehen, falls keine da sind und sie nicht identisch sind.
         */
        for (Vertex backwardsNeighbour : backwardNeighbours) {
            for (Vertex forewardsNeighbour : forwardNeighbours) {
                if (forewardsNeighbour.equals(backwardsNeighbour)) {
                    continue;
                }
                if (existsEdge(backwardsNeighbour, forewardsNeighbour)) {
                    continue;
                }
                addEdge(backwardsNeighbour, forewardsNeighbour);
            }
        }
    }

    private void insertNewEdgesWhileMergingVertexInUndirectedGraph(int index) {
        /* 1) Alle Nachbarn ermitteln: */
        List<Vertex> neighbours = getForwardNeighboursAsVertices(index);

        /* 2) Alle paarweise miteinander verbinden, falls noch keine Kante da ist: */
        for (int index1 = 0; index1 < neighbours.size() - 1; ++index1) {
            Vertex vertex1 = neighbours.get(index1);
            for (int index2 = index1 + 1; index2 < neighbours.size(); ++index2) {
                Vertex vertex2 = neighbours.get(index2);
                if (!existsEdge(vertex1, vertex2)) {
                    addEdge(vertex1, vertex2);
                }
            }
        }
    }

    /**
     * Prüft, ob die übergebene Kante existiert. Berücksichtigt werden hier nur der Start- und er
     * Ziel-Ecken-Index.
     *
     * @param edge
     *            Kante.
     */
    public boolean existsEdge(Edge edge) {
        int startVertexIndex = edge.getStartVertexIndex();
        int targetVertexIndex = edge.getTargetVertexIndex();
        return existsEdge(startVertexIndex, targetVertexIndex);
    }

    /**
     * Prüft, ob eine Kante von der startVertex zur endVertex existiert.
     *
     * @param startVertex
     *            Startecke der Kante.
     * @param endVertex
     *            Zielecke der Kante.
     */
    public boolean existsEdge(Vertex startVertex, Vertex endVertex) {
        int startVertexIndex = startVertex.getIndex();
        int targetVertexIndex = endVertex.getIndex();
        return existsEdge(startVertexIndex, targetVertexIndex);
    }

    /**
     * Prüft, ob eine Kante von der startVertex zur endVertex existiert.
     *
     * @param startVertexIndex
     *            Index der Startecke der Kante.
     * @param endEndex
     *            Index der Zielecke der Kante.
     */
    public boolean existsEdge(int startVertexIndex, int targetVertexIndex) {
        for (Edge edge : edges) {
            if (startVertexIndex == edge.getStartVertexIndex()
                    && targetVertexIndex == edge.getTargetVertexIndex()) {
                return true;
            }
            if (isUndirected()
                    && targetVertexIndex == edge.getStartVertexIndex()
                    && startVertexIndex == edge.getTargetVertexIndex()) {
                return true;
            }
        }

        return false;
    }

    /**
     * Verschmilzt die Ecke mit dem gegebenen Index.
     *
     * @param vertex
     *            Zu verschmelzende Ecke.
     */
    public void mergeVertex(Vertex vertex) {
        mergeVertex(vertex.getIndex());
    }

    /** Liefert die Liste mit den Ecken zurück. */
    public Vertices getVertices() {
        return vertices;
        // TODO Verwendung diese Methode vermeiden, alles was anderswo damit gemacht wird, in Graph
        // einbauen. "Tell, don't ask"
    }

    /**
     * Liefert die Ecke zum gegebenen Index zurück.
     *
     * @param index
     *            Index der gesuchten Ecke.
     * @return Gesuchte Ecke.
     */
    public Vertex getVertex(int index) {
        return vertices.get(index);
    }

    /** Liefert die Liste mit den Kanten zurück. */
    public Edges getEdges() {
        return edges;
        // TODO Verwendung diese Methode vermeiden, alles was anderswo damit gemacht wird, in Graph
        // einbauen. "Tell, don't ask"
    }

    /**
     * Liefert die Kante zum gegebenen Index zurück.
     *
     * @param index
     *            Index der gesuchten Kante.
     * @return Gesuchte Kante.
     */
    public Edge getEdge(int index) {
        return edges.get(index);
    }

    public Edge getEdge(int startVertexIndex, int targetVertexIndex) {
        for (Edge edge : edges) {
            if (edge.getStartVertexIndex() == startVertexIndex
                    && edge.getTargetVertexIndex() == targetVertexIndex) {
                return edge;
            }
        }
        if (isUndirected()) {
            for (Edge edge : edges) {
                if (edge.getStartVertexIndex() == targetVertexIndex
                        && edge.getTargetVertexIndex() == startVertexIndex) {
                    return edge;
                }
            }
        }
        throw new RuntimeException("Es gibt keine Kante vom Index " + startVertexIndex + " nach "
                + targetVertexIndex + "!");
    }

    /**
     * Gibt die Liste mit den von der Ecke ausgehenden Kanten zurück.
     *
     * @param vertex
     *            Ausgangsecke
     * @return Liste mit den zu Nachbarn führenden Kanten.
     */
    public Edges getEdgesStartingFromVertex(Vertex vertex) {
        return getEdgesStartingFromVertex(vertex.getIndex());
    }

    /**
     * Gibt die Liste mit den von der Ecke ausgehenden Kanten zurück.
     *
     * @param vertexIndex
     *            Index der Ausgangsecke.
     * @return Liste mit den zu Nachbarn führenden Kanten.
     */
    public Edges getEdgesStartingFromVertex(int vertexIndex) {
        Edges edgesStartingFromVertex = new Edges();
        for (Edge edge : edges) {
            if (edge.getStartVertexIndex() == vertexIndex) {
                edgesStartingFromVertex.add(edge);
            }
        }
        return edgesStartingFromVertex;
    }

    /**
     * Gibt die Liste mit den zu der Ecke führenden Kanten zurück.
     *
     * @param vertex
     *            Zielecke.
     * @return Liste mit von den Nachbarn zur Ecke führenden Kanten.
     */
    public Edges getEdgesTargetingVertex(Vertex vertex) {
        return getEdgesTargetingVertex(vertex.getIndex());
    }

    /**
     * Gibt die Liste mit den zu der Ecke führenden Kanten zurück.
     *
     * @param vertexIndex
     *            Index der Zielecke.
     * @return Liste mit von den Nachbarn zur Ecke führenden Kanten.
     */
    public Edges getEdgesTargetingVertex(int vertexIndex) {
        Edges edgesTargetingVertex = new Edges();
        for (Edge edge : edges) {
            if (edge.getTargetVertexIndex() == vertexIndex) {
                edgesTargetingVertex.add(edge);
            }
        }
        return edgesTargetingVertex;
    }

    /**
     * Gibt die Nachbarn als Liste von Ecken zurück.
     *
     * @param vertex
     *            Ausgangsecke
     * @return Liste mit den Nachbarn.
     */
    public List<Vertex> getForwardNeighboursAsVertices(Vertex vertex) {
        return getForwardNeighboursAsVertices(vertex.getIndex());
    }

    /**
     * Gibt die Nachbarn als Liste von Ecken zurück.
     *
     * @param index
     *            Index der Ecke.
     * @return Liste mit den Nachbarn.
     */
    private List<Vertex> getForwardNeighboursAsVertices(int index) {
        List<Vertex> neighbours = new ArrayList<>();

        Edges edgesStartingFromVertex = getEdgesStartingFromVertex(index);
        for (Edge edge : edgesStartingFromVertex) {
            neighbours.add(getVertex(edge.getTargetVertexIndex()));
        }

        if (isUndirected()) {
            Edges edgesTargetingVertex = getEdgesTargetingVertex(index);
            for (Edge edge : edgesTargetingVertex) {
                neighbours.add(getVertex(edge.getStartVertexIndex()));
            }
        }

        return neighbours;
    }

    /**
     * Gibt die Ecken, von denen Kanten zu dieser Ecke hin laufen, als Liste von Ecken zurück.
     *
     * @param vertex
     *            Ausgangsecke
     * @return Liste mit den Nachbarn.
     */
    public List<Vertex> getBackwardNeighboursAsVertices(Vertex vertex) {
        return getBackwardNeighboursAsVertices(vertex.getIndex());
    }

    /**
     * Gibt die Ecken, von denen Kanten zu dieser Ecke hin laufen, als Liste von Ecken zurück.
     *
     * @param index
     *            Index der Ecke.
     * @return Liste mit den Nachbarn.
     */
    private List<Vertex> getBackwardNeighboursAsVertices(int index) {
        List<Vertex> neighbours = new ArrayList<>();

        if (isUndirected()) {
            Edges edgesStartingFromVertex = getEdgesStartingFromVertex(index);
            for (Edge edge : edgesStartingFromVertex) {
                neighbours.add(getVertex(edge.getTargetVertexIndex()));
            }
        }

        Edges edgesTargetingVertex = getEdgesTargetingVertex(index);
        for (Edge edge : edgesTargetingVertex) {
            neighbours.add(getVertex(edge.getStartVertexIndex()));
        }

        return neighbours;
    }

    /** Getter für das Objekt mit den Metadaten. */
    public MetaData getMeta() {
        return meta;
    }

    /** Setter für das Objekt mit den Metadaten. */
    public void copyMeta(MetaData meta) {
        this.meta.copyValues(meta);
    }

    /**
     * Lädt den Graphen aus dem .ged-Format und ersetzt den aktuellen Graphen
     * durch den geladenen.
     *
     * @param text
     *            Eingelesene Datei als ein String.
     */
    public void loadFromGed1Format(String text) {
        Ged1FormatLoader loader = new Ged1FormatLoader(this);
        loader.loadFromSaveString(text);
    }


    /** Gibt den Graphen als String gespeichert im ged-Format zurück. */
    public String saveToGed1Format() {
        Ged1FormatSaver saver = new Ged1FormatSaver(this);
        return saver.createSaveString();
    }

    /**
     * Lädt den Graphen aus dem .ged2-Format und ersetzt den aktuellen Graphen
     * durch den geladenen.
     *
     * @param text
     *            Eingelesene Datei als ein String.
     */
    public void loadFromGed2Format(String text) {
        Ged2FormatLoader loader = new Ged2FormatLoader(this);
        loader.loadFromSaveString(text);
    }


    /** Gibt den Graphen als String gespeichert im ged2-Format zurück. */
    public String saveToGed2Format() {
        Ged2FormatSaver saver = new Ged2FormatSaver(this);
        return saver.createSaveString();
    }

    /**
     * Lädt den Graphen aus dem .ged3-Format und ersetzt den aktuellen Graphen
     * durch den geladenen.
     *
     * @param text
     *            Eingelesene Datei als ein String.
     */
    public void loadFromGed3Format(String text) {
        Ged3FormatLoader loader = new Ged3FormatLoader(this);
        loader.loadFromSaveString(text);
    }


    /** Gibt den Graphen als String gespeichert im ged3-Format zurück. */
    public String saveToGed3Format() {
        Ged3FormatSaver saver = new Ged3FormatSaver(this);
        return saver.createSaveString();
    }

    /**
     * Lädt den Graphen aus dem .ged4-Format und ersetzt den aktuellen Graphen
     * durch den geladenen.
     *
     * @param text
     *            Eingelesene Datei als ein String.
     */
    public void loadFromGed4Format(String text) {
        Ged4FormatLoader loader = new Ged4FormatLoader(this);
        loader.loadFromSaveString(text);
    }

    /** Gibt den Graphen als String gespeichert im ged4-Format zurück. */
    public String saveToGed4Format() {
        Ged4FormatSaver saver = new Ged4FormatSaver(this);
        return saver.createSaveString();
    }

    /** Löscht alle Daten und setzt den Graphen auf den leeren Graphen zurück. */
    public void clearGraph() { // TODO per Interfaces nur den Lademethoden zur Verfügung stellen.
        meta.clear();
        vertices.clear();
        edges.clear();
    }

    /** Bewegt den Graphen um den Gitterabstand nach links. */
    public void moveLeft() {
        move(-meta.getGridDistance(), 0);
    }

    /** Bewegt den Graphen um den Gitterabstand nach rechts. */
    public void moveRight() {
        move(meta.getGridDistance(), 0);
    }

    /** Bewegt den Graphen um den Gitterabstand nach oben. */
    public void moveUp() {
        move(0, -meta.getGridDistance());
    }

    /** Bewegt den Graphen um den Gitterabstand nach unten. */
    public void moveDown() {
        move(0, meta.getGridDistance());
    }

    /** Bewegt den Graphen um die angegebene Anzahl von Pixeln. */
    private void move(int xDistance, int yDistance) {
        for (Vertex vertex : vertices) {
            PixelPoint coord = vertex.getCoordinates();
            int x = coord.getX() + xDistance;
            int y = coord.getY() + yDistance;
            coord.setX(x);
            coord.setY(y);
        }
    }

    /**
     * Zentriert den Graphen auf den Mittelpunkt der sichtbaren Fläche.
     *
     * @param minX
     *            x-Koordinate der linken oberen Ecke der Fläche.
     * @param minY
     *            y-Koordinate der linken oberen Ecke der Fläche.
     * @param maxX
     *            x-Koordinate der rechten unteren Ecke der Fläche.
     * @param maxY
     *            y-Koordinate der rechten unteren Ecke der Fläche.
     */
    public void centerGraph(int minX, int minY, int maxX, int maxY) {
        if (vertices.isEmpty()) {
            return;
        }

        /* Mittelpunkt der Fläche berechnen: */
        int centerX = (int) ((maxX + minX) / 2.0);
        int centerY = (int) ((maxY + minY) / 2.0);

        /* Extremale Koordinaten des Graphen bestimmen: */
        int vertexMinX = calculateMinimalX();
        int vertexMinY = calculateMinimalY();
        int vertexMaxX = calculateMaximalX();
        int vertexMaxY = calculateMayimalY();

        /* Mittelpunkt des Graphen berechnen: */
        double vertexCenterX = (vertexMaxX + vertexMinX) / 2.0;
        double vertexCenterY = (vertexMaxY + vertexMinY) / 2.0;

        /* Koordinatentransformation: */
        double addX = centerX - vertexCenterX;
        double addY = centerY - vertexCenterY;
        for (Vertex vertex : vertices) {
            PixelPoint p = vertex.getCoordinates();
            int x = p.getX();
            int y = p.getY();
            x += addX;
            y += addY;
            p.setX(x);
            p.setY(y);
        }
    }

    /** Berechnet den kleinsten x-Koordinaten-Wert aller Ecken. */
    private int calculateMinimalX() {
        int minX = Integer.MAX_VALUE;
        for (Vertex vertex : vertices) {
            PixelPoint p = vertex.getCoordinates();
            int x = p.getX();
            if (x < minX) {
                minX = x;
            }
        }
        return minX;
    }

    /** Berechnet den größten x-Koordinaten-Wert aller Ecken. */
    private int calculateMaximalX() {
        int maxX = Integer.MIN_VALUE;
        for (Vertex vertex : vertices) {
            PixelPoint p = vertex.getCoordinates();
            int x = p.getX();
            if (x > maxX) {
                maxX = x;
            }
        }
        return maxX;
    }

    /** Berechnet den kleinsten y-Koordinaten-Wert aller Ecken. */
    private int calculateMinimalY() {
        int minY = Integer.MAX_VALUE;
        for (Vertex vertex : vertices) {
            PixelPoint p = vertex.getCoordinates();
            int y = p.getY();
            if (y < minY) {
                minY = y;
            }
        }
        return minY;
    }

    /** Berechnet den größten y-Koordinaten-Wert aller Ecken. */
    private int calculateMayimalY() {
        int maxY = Integer.MIN_VALUE;
        for (Vertex vertex : vertices) {
            PixelPoint p = vertex.getCoordinates();
            int y = p.getY();
            if (y > maxY) {
                maxY = y;
            }
        }
        return maxY;
    }

    /** Berechnet den größten Radius aller Ecken. */
    private int calculateMinimalRadius() {
        int maxRadius = Integer.MIN_VALUE;
        for (Vertex vertex : vertices) {
            int radius = vertex.getRadius();
            if (maxRadius < radius) {
                maxRadius = radius;
            }
        }

        if (maxRadius == Vertex.USE_DEFAULT_RADIUS) {
            maxRadius = meta.getDefaultRadius();
        }

        return maxRadius;
    }

    /**
     * Zentriert den Graphen auf den Mittelpunkt der sichtbaren Fläche und vergrößert ihn so weit
     * er dann noch auf den Schirm passt.
     *
     * @param minX
     *            x-Koordinate der linken oberen Ecke der Fläche.
     * @param minY
     *            y-Koordinate der linken oberen Ecke der Fläche.
     * @param maxX
     *            x-Koordinate der rechten unteren Ecke der Fläche.
     * @param maxY
     *            y-Koordinate der rechten unteren Ecke der Fläche.
     */
    public void maximizeGraph(int minX, int minY, int maxX, int maxY) {
        boolean respectingAspectRatio = false;
        boolean centerAroundVertex = false;
        Vertex unusedCenter = new Vertex();
        maximizeGraph(minX, minY, maxX, maxY, respectingAspectRatio, centerAroundVertex,
                unusedCenter);
    }

    /**
     * Zentriert den Graphen auf den Mittelpunkt der sichtbaren Fläche und vergrößert ihn so weit
     * er dann noch auf den Schirm passt.
     *
     * @param minX
     *            x-Koordinate der linken oberen Ecke der Fläche.
     * @param minY
     *            y-Koordinate der linken oberen Ecke der Fläche.
     * @param maxX
     *            x-Koordinate der rechten unteren Ecke der Fläche.
     * @param maxY
     *            y-Koordinate der rechten unteren Ecke der Fläche.
     */
    public void maximizeGraphRespectingAspectRatio(int minX, int minY, int maxX, int maxY) {
        boolean respectingAspectRatio = true;
        boolean centerAroundVertex = false;
        Vertex unusedCenter = new Vertex();
        maximizeGraph(minX, minY, maxX, maxY, respectingAspectRatio, centerAroundVertex,
                unusedCenter);
    }

    /**
     * Zentriert den Graphen auf den Mittelpunkt der sichtbaren Fläche und vergrößert ihn so weit
     * er dann noch auf den Schirm passt.
     *
     * @param minX
     *            x-Koordinate der linken oberen Ecke der Fläche.
     * @param minY
     *            y-Koordinate der linken oberen Ecke der Fläche.
     * @param maxX
     *            x-Koordinate der rechten unteren Ecke der Fläche.
     * @param maxY
     *            y-Koordinate der rechten unteren Ecke der Fläche.
     * @param center
     *            Ecke die in den Mittelpunkt gerückt wird.
     */
    public void maximizeGraphRespectingAspectRatio(int minX, int minY, int maxX, int maxY,
            Vertex center) {
        boolean respectingAspectRatio = true;
        boolean centerAroundVertex = true;
        maximizeGraph(minX, minY, maxX, maxY, respectingAspectRatio, centerAroundVertex, center);
    }

    private void maximizeGraph(int minX, int minY, int maxX, int maxY,
            boolean respectingAspectRatio, boolean centerAroundVertex, Vertex center) {
        if (vertices.isEmpty()) {
            return;
        }

        int maxRadius = calculateMinimalRadius();

        /* Extreme erlaubte Werte bestimmen: */
        int delta = 20;
        if (maxRadius >= delta - 2) {
            delta = maxRadius + 10;
            // TODO Beschriftungen und so weiter können trotzdem außerhalb liegen.
        }
        int screenMinX = minX + delta;
        int screenMinY = minY + delta;
        int screenMaxX = maxX - delta;
        int screenMaxY = maxY - delta;

        /* Extremale Koordinaten des Graphen bestimmen: */
        int vertexMinX = calculateMinimalX();
        int vertexMinY = calculateMinimalY();
        int vertexMaxX = calculateMaximalX();
        int vertexMaxY = calculateMayimalY();

        if (centerAroundVertex) {
            int middleX = vertexMinX + (vertexMaxX - vertexMinX) / 2;
            int middleY = vertexMinY + (vertexMaxY - vertexMinY) / 2;

            int wantedCenterX = center.getX();
            int wantedCenterY = center.getY();

            int moveX = wantedCenterX - middleX;
            int moveY = wantedCenterY - middleY;

            if (moveX > 0) { // TODO eventuell * 2 überall ...
                vertexMaxX += 2 * moveX;
            }
            else {
                vertexMinX += 2 * moveX; // moveX ist dann ja negativ.
            }
            if (moveY > 0) {
                vertexMaxY += 2 * moveY;
            }
            else {
                vertexMinY += 2 * moveY; // moveY ist dann ja negativ.
            }
        }

        /* Berechnung der Faktoren für die Koordinatentransformation: */
        double xFactor;
        if (vertexMaxX == vertexMinX) {
            xFactor = 1.0;
        }
        else {
            xFactor = (screenMaxX - screenMinX) / (1.0 * vertexMaxX - vertexMinX);
        }
        double yFactor;
        if (vertexMaxY == vertexMinY) {
            yFactor = 1.0;
        }
        else {
            yFactor = (screenMaxY - screenMinY) / (1.0 * vertexMaxY - vertexMinY);
        }

        if (respectingAspectRatio) {
            double minFactor = Math.min(xFactor, yFactor);
            xFactor = minFactor;
            yFactor = minFactor;
        }

        /* Koordinatentransformation: */
        for (Vertex vertex : vertices) {
            PixelPoint p = vertex.getCoordinates();
            double x = p.getX();
            double y = p.getY();
            if (vertexMaxX == vertexMinX) {
                x = (screenMinX + screenMaxX) / 2.0;
            }
            else {
                x = screenMinX + (x - vertexMinX) * xFactor;
            }
            if (vertexMaxY == vertexMinY) {
                y = (screenMinY + screenMaxY) / 2.0;
            }
            else {
                y = screenMinY + (y - vertexMinY) * yFactor;
            }
            p.setX(x);
            p.setY(y);
        }

        /* Gitterweite mit verändern: */
        double distance = meta.getGridDistance();
        if (xFactor > yFactor) {
            distance *= xFactor;
        }
        else {
            distance *= yFactor;
        }
        meta.setGridDistance((int) distance);

        /*
         * Auf automatisches Einrasten an dieser Stelle wird verzichtet,
         * da der Benutzer dies selbst machen kann, wenn gewünscht.
         */
    }

    /**
     * Verkleinert den Graphen.
     *
     * @param minX
     *            x-Koordinate der linken oberen Ecke der Fläche.
     * @param minY
     *            y-Koordinate der linken oberen Ecke der Fläche.
     * @param maxX
     *            x-Koordinate der rechten unteren Ecke der Fläche.
     * @param maxY
     *            y-Koordinate der rechten unteren Ecke der Fläche.
     */
    public void reduceGraph(int minX, int minY, int maxX, int maxY) {
        if (vertices.isEmpty()) {
            return;
        }

        /* Extremale Koordinaten des Graphen bestimmen: */
        int vertexMinX = calculateMinimalX();
        int vertexMinY = calculateMinimalY();
        int vertexMaxX = calculateMaximalX();
        int vertexMaxY = calculateMayimalY();

        /* Mittelpunkt des Graphen berechnen: */
        double vertexCenterX = (vertexMaxX + vertexMinX) / 2.0;
        double vertexCenterY = (vertexMaxY + vertexMinY) / 2.0;

        /* Koordinatentransformation: */
        double factor = 0.8; // 0.66666
        for (Vertex vertex : vertices) {
            PixelPoint p = vertex.getCoordinates();
            double x = p.getX();
            double y = p.getY();
            x = vertexCenterX + (x - vertexCenterX) * factor;
            y = vertexCenterY + (y - vertexCenterY) * factor;
            p.setX(x);
            p.setY(y);
        }

        /* Gitterweite mit verändern: */
        double distance = meta.getGridDistance();
        distance *= factor;
        meta.setGridDistance((int) distance);
    }

    /** Vergrößert den Graphen.
     *
     * @param minX
     *            x-Koordinate der linken oberen Ecke der Fläche.
     * @param minY
     *            y-Koordinate der linken oberen Ecke der Fläche.
     * @param maxX
     *            x-Koordinate der rechten unteren Ecke der Fläche.
     * @param maxY
     *            y-Koordinate der rechten unteren Ecke der Fläche.
     */
    public void enlargeGraph(int minX, int minY, int maxX, int maxY) {
        if (vertices.isEmpty()) {
            return;
        }
        /* Extremale Koordinaten des Graphen bestimmen: */
        int vertexMinX = calculateMinimalX();
        int vertexMinY = calculateMinimalY();
        int vertexMaxX = calculateMaximalX();
        int vertexMaxY = calculateMayimalY();

        /* Mittelpunkt des Graphen berechnen: */
        double vertexCenterX = (vertexMaxX + vertexMinX) / 2.0;
        double vertexCenterY = (vertexMaxY + vertexMinY) / 2.0;

        /* Koordinatentransformation: */
        double factor = 1.25; // 1.5
        for (Vertex vertex : vertices) {
            PixelPoint p = vertex.getCoordinates();
            double x = p.getX();
            double y = p.getY();
            x = vertexCenterX + (x - vertexCenterX) * factor;
            y = vertexCenterY + (y - vertexCenterY) * factor;
            p.setX(x);
            p.setY(y);
        }

        /* Gitterweite mit verändern: */
        double distance = meta.getGridDistance();
        distance *= factor;
        meta.setGridDistance((int) distance);
    }

    /** Lässt alle Ecken des Graphen im Gitter einrasten. */
    public void snapAllVerticesToGrid() {
        for (Vertex vertex : vertices) {
            snapVertexToGrid(vertex);
        }
    }

    /**
     * Testet, ob diese Ecke sich in ihrer Darstellung mit einer anderen Ecke überschneidet.
     *
     * @param vertex
     *            Zu prüfende Ecke.
     * @return Wahrheitswert true bedeutet, sie überschneidet sich mit mindestens einer anderen
     *         Ecke.
     */
    public boolean vertexIntersctsWithOtherVertex(Vertex vertex) {
        for (Vertex otherVertex : vertices) {
            /* Kein Vergleich mit sich selbst! */
            if (vertex != otherVertex) {
                /*
                 * Wir prüfen, ob die Summe der Radien kürzer oder so lang ist wie
                 * Verbindungslinien zwischen den zwei Ecken:
                 */
                int ourRadius = vertex.getRadius();
                int otherRadius = otherVertex.getRadius();
                PixelPoint ourCenter = vertex.getCoordinates();
                PixelPoint otherCenter = otherVertex.getCoordinates();
                Line line = new Line(ourCenter.toMathPoint(), otherCenter.toMathPoint());
                if (line.getDistance() <= ourRadius + otherRadius) {
                    return true;
                }
            }
        }
        return false;
    }

    /** Setzt die Farben aller Ecken auf Default. */
    public void setAllVertexColorsToDefault() {
        for (Vertex vertex : vertices) {
            vertex.setColor(GedColor.DEFAULT);
        }
    }

    /** Setzt die Radien aller Ecken auf Default. */
    public void setAllVertexRadiusToDefault() {
        for (Vertex vertex : vertices) {
            vertex.setRadiusToDefault();
        }
    }

    /** Setzt die Farben aller Kanten auf Default. */
    public void setAllEdgeColorsToDefault() {
        for (Edge edge : edges) {
            edge.setColorToDefault();
        }
    }

    /** Setzt die Dicken aller Kanten auf Default. */
    public void setAllEdgeWidthsToDefault() {
        for (Edge edge : edges) {
            edge.setWidthToDefault();
        }
    }

    /**
     * Erzeugt einen quadratischen Teilgraphen mit einer Kante.
     *
     * @param edge
     *            Darzustellende Kante.
     * @param subGraphSize
     *            Breite und Höhe des Teilgraphen.
     */
    public Graph createEdgeSubGraph(Edge egde, int subGraphSize) {
        return createEdgeSubGraph(egde, subGraphSize, subGraphSize);
    }

    /**
     * Erzeugt einen rechteckigen Teilgraphen mit einer Kante.
     *
     * @param edge
     *            Darzustellende Kante.
     * @param subGraphSizeX
     *            Breite des Teilgraphen.
     * @param subGraphSizeY
     *            Höhe des Teilgraphen.
     */
    public Graph createEdgeSubGraph(Edge edge, int subGraphSizeX, int subGraphSizeY) {
        EdgeSubGraphCreator creator = new EdgeSubGraphCreator(this, subGraphSizeX, subGraphSizeY);
        return creator.createOneEdgeSubGraph(edge);
    }

    /**
     * Erzeugt einen quadratischen Teilgraphen mit der angegebenen Ecke und ihren Nachbarn.
     *
     * @param vertex
     *            Darzustellende Ecke.
     * @param subGraphSize
     *            Breite und Höhe des Teilgraphen.
     */
    public Graph createVertexWithNeighboursSubGraph(Vertex vertex, int subGraphSize) {
        return createVertexWithNeighboursSubGraph(vertex.getIndex(), subGraphSize);
    }

    /**
     * Erzeugt einen quadratischen Teilgraphen mit der angegebenen Ecke und ihren Nachbarn.
     *
     * @param vertexIndex
     *            Index der darzustellenden Ecke.
     * @param subGraphSize
     *            Breite und Höhe des Teilgraphen.
     */
    public Graph createVertexWithNeighboursSubGraph(int vertexIndex, int subGraphSize) {
        return createVertexWithNeighboursSubGraph(vertexIndex, subGraphSize, subGraphSize);
    }

    /**
     * Erzeugt einen rechteckigen Teilgraphen mit der angegebenen Ecke und ihren Nachbarn.
     *
     * @param vertexIndex
     *            Index der darzustellenden Ecke.
     * @param subGraphSizeX
     *            Breite des Teilgraphen.
     * @param subGraphSizeY
     *            Höhe des Teilgraphen.
     */
    public Graph createVertexWithNeighboursSubGraph(int vertexIndex, int subGraphSizeX,
            int subGraphSizeY) {
        VertexWithItsNeigboursSubGraphCreator creator = new VertexWithItsNeigboursSubGraphCreator(
                this, subGraphSizeX, subGraphSizeY);
        return creator.createVertexWithItsNeigboursSubGraph(vertexIndex);
    }

    /** Gibt an, ob der Graph gerichtet ist. */
    public boolean isDirected() {
        return meta.isDirected();
    }

    /** Gibt an, ob der Graph ungerichtet ist. */
    public boolean isUndirected() {
        return !isDirected();
    }

    /** Legt fest, ob der Graph gerichtet ist. */
    public void setDirected(boolean directed) {
        if (directed && isDirected() || !directed && isUndirected()) {
            throw new RuntimeException("Interner Logikfeler gerichtet / ungerichtet!");
        }

        if (isDirected()) {
            transformToUndirected();
        }
        else {
            transformToDirected();
        }

        meta.setDirected(directed);
    }

    /* Wandelt den Graphen um, noch ist er gerichtet! */
    private void transformToUndirected() {
        /*
         * Wir müssen prüfen, ob nicht zwei vorher verschiedene gerichtete Kanten nun zu einer
         * "identischen" ungerichteten werden:
         */
        boolean deleting = true;
        while (deleting) {
            deleting = false;
            for (Edge edge : edges) {
                int startVertexIndex = edge.getStartVertexIndex();
                int targetVertexIndex = edge.getTargetVertexIndex();

                if (existsEdge(targetVertexIndex, startVertexIndex)) {
                    removeEdge(targetVertexIndex, startVertexIndex);
                    deleting = true;
                    break;
                }
            }
        }
    }

    /* Wandelt den Graphen um, noch ist er ungerichtet! */
    private void transformToDirected() {
        /*
         * Hier müssen wir nichts tun, dir Kanten haben nun eine zufällige Richtung.
         */
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((edges == null) ? 0 : edges.hashCode());
        result = prime * result + ((meta == null) ? 0 : meta.hashCode());
        result = prime * result + ((vertices == null) ? 0 : vertices.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        Graph other = (Graph) obj;
        if (edges == null) {
            if (other.edges != null)
                return false;
        }
        else if (!edges.equals(other.edges))
            return false;
        if (meta == null) {
            if (other.meta != null)
                return false;
        }
        else if (!meta.equals(other.meta))
            return false;
        if (vertices == null) {
            if (other.vertices != null)
                return false;
        }
        else if (!vertices.equals(other.vertices))
            return false;
        return true;
    }

}
