package de.duehl.math.graph.ged.logic;

import java.awt.Desktop;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import javax.imageio.ImageIO;

import de.duehl.basics.exceptions.WrongSourceFormatException;
import de.duehl.basics.history.History;
import de.duehl.basics.history.HistoryHelper;
import de.duehl.math.graph.ged.graph.GedColor;
import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.graph.MetaData;
import de.duehl.math.graph.ged.graph.edge.Edge;
import de.duehl.math.graph.ged.graph.edge.Edges;
import de.duehl.math.graph.ged.graph.vertex.Vertex;
import de.duehl.math.graph.ged.logic.ObserverPattern.Observable;
import de.duehl.math.graph.ged.logic.ObserverPattern.Observer;
import de.duehl.math.graph.ged.logic.history.GraphCopier;
import de.duehl.math.graph.ged.pixel.PixelPoint;
import de.duehl.math.graph.ged.ui.GedGui;
import de.duehl.math.graph.ged.ui.Gui;
import de.duehl.math.graph.ged.ui.graphpanel.GraphPanel;
import de.duehl.swing.ui.GuiTools;
import de.duehl.basics.io.FileHelper;
import de.duehl.basics.logging.FileLogger;
import de.duehl.basics.logging.Logger;
import de.duehl.basics.logic.Version;

/**
 * Diese Klasse steht für die Logik des Programms.
 *
 * @version 1.01     2019-07-02
 * @author Christian Dühl
 */

public class GedLogic implements Observer, Logic {

    private static final int START_GRAPH_PANEL_WIDTH = 600;
    private static final int START_GRAPH_PANEL_HEIGHT = 450;

    /** Verbindung zur Gui. */
    private final Gui gui;

    /** Hält die veränderten Graphen der History vor. */
    private final History<Graph> history;

    /** Programmversion. */
    private final Version version;

    /** Der Logger. */
    private final Logger logger;

    /** Name, unter der der Graph gespeichert wird. */
    private String filename;

    /** Das Verzeichnis ~/.ged für Einstellungen und Graphendateien. */
    private final String directory;

    /** Konstruktor */
    public GedLogic(Version version) {
        this.version = version;

        directory = FileHelper.getAndPerhapsCreateDirectoryAtHome(".ged");
        String logDir = FileHelper.concatPathes(directory, "logs");
        FileHelper.createDirectoryIfNotExists(logDir);
        logger = FileLogger.create(".gedLog", ".log", logDir);

        Graph graph = new Graph(); // Die Gui hält den Graphen, weil sie so viel damit anstellt.

        history = new History<>(new GraphCopier());
        history.add(graph);

        gui = new GedGui(START_GRAPH_PANEL_WIDTH, START_GRAPH_PANEL_HEIGHT, this);
        gui.setGraph(graph);
        logger.log("Gui created");
        filename = "";

        Observable observable = gui.getGraphPanel();
        observable.addObserver(this);

        logger.log("GedLogic constructor finished");
        gui.setVisible(true);
    }

    /** Zum Mitteilen von Änderungen am Graphen. */
    @Override
    public void update() {
        gui.setChanged();
        history.add(gui.getGraph());
        checkEnableHistory();
        setStatus("");
    }

    /** Erzeugt einen neuen, leeren Graphen. */
    @Override
    public void createEmptyFile() {
        Graph emptyGraph = new Graph();
        gui.setGraph(emptyGraph);
        update();
        logger.log("empty graph created");
    }

    /** Beendet das Programm. */
    @Override
    public void quitProgram() {
        boolean quit = gui.askUserToQuit();
        if (quit) {
            logger.log("normal program termination");
            System.exit(0);
        }
    }

    /** Fragt nach dem zu ladenden Dateinamen und lädt diesen neuen Graphen aus dem .ged Format. */
    @Override
    public void loadGedFile() {
        // TODO : implement
        GuiTools.informUser("Sorry, noch nicht Implementiert...",
                "Das Laden des Graphen im .ged-Format ist noch nicht implementiert.");
        logger.log("load ged file");
    }

    /** Fragt nach dem zu ladenden Dateinamen und lädt diesen neuen Graphen aus dem .ged2-Format. */
    @Override
    public void loadGed2File() {
        // TODO : implement
        GuiTools.informUser("Sorry, noch nicht Implementiert...",
                "Das Laden des Graphen im .ged2-Format ist noch nicht implementiert.");
        logger.log("load ged2 file");
    }

    /** Fragt nach dem zu ladenden Dateinamen und lädt diesen neuen Graphen aus dem .ged3-Format. */
    @Override
    public void loadGed3File() {
        String filename = gui.askUserToOpenAsGed3();
        if (filename.isEmpty()) {
            setStatus("Abbruch durch den Benutzer.");
            return;
        }

        if (filename.endsWith(".ged3")) {
            loadGed3File(filename);
            setStatus("Datei '" + filename + "' erfolgreich geladen.");
            // Hier wird der Dateiname nicht abgelegt, da es ein altes Format ist.
            logger.log("load ged3 file '" + filename + "'");
        }
        else {
            gui.error("Die Datei '" + filename + "' liegt nicht im .ged3-Format vor.");
        }
    }

    /**
     * Lädt einen neuen Graphen aus dem .ged3-Format.
     *
     * @param filename
     *            Datei aus der der Graph geladen werden soll.
     */
    private void loadGed3File(String filename) {
        gui.startLongTimeProcess("Lade Datei im GED3-Format...");
        String fullContents = FileHelper.readFileToString(filename);
        try {
            Graph graph = new Graph();
            graph.loadFromGed3Format(fullContents);
            gui.setGraph(graph);
            setStatus("Datei '" + filename + "' erfolgreich geladen.");
        }
        catch (WrongSourceFormatException | NumberFormatException exception) {
            gui.error("Fehler beim Laden des Graphen " + filename + " im GED3-Format.", exception);
        }
        update();
        gui.endLongTimeProcess();
    }

    /** Fragt nach dem zu ladenden Dateinamen und lädt diesen neuen Graphen aus dem .ged4-Format. */
    @Override
    public void loadGed4File() {
        String filename = gui.askUserToOpenAsGed4();
        if (filename.isEmpty()) {
            setStatus("Abbruch durch den Benutzer.");
            return;
        }

        if (filename.endsWith(".ged4")) {
            loadGed4File(filename);
        }
        else {
            gui.error("Die Datei '" + filename + "' liegt nicht im .ged4-Format vor.");
        }
    }

    /**
     * Lädt einen neuen Graphen aus dem .ged4-Format.
     *
     * @param file
     *            Datei aus der der Graph geladen werden soll.
     */
    private void loadGed4File(String filename) {
        gui.startLongTimeProcess("Lade Datei im GED4-Format...");
        String fullContents = FileHelper.readFileToString(filename);
        try {
            Graph graph = new Graph();
            graph.loadFromGed4Format(fullContents);
            gui.setGraph(graph);
            this.filename = filename;
            setStatus("Datei '" + filename + "' erfolgreich geladen.");
            gui.showFileName(filename);
            logger.log("load ged4 file '" + filename + "'");
        }
        catch (WrongSourceFormatException | NumberFormatException exception) {
            gui.error("Fehler beim Laden des Graphen " + filename + " im GED4-Format.", exception);
        }
        update();
        gui.endLongTimeProcess();
    }

    /** Setzt den angezeigten Statustext. */
    @Override
    public void setStatus(String statusText) {
        if (null != gui) {
            gui.setStatus(statusText);
        }
    }

    /** Unterbricht eine ggf. begonnene Bearbeitung. */
    @Override
    public void interruptEditing() {
        gui.interruptEditing();
    }

    /** Speichert den Graphen im .ged-Format. */
    @Override
    public void saveGedFile() {
        // TODO : implement
        GuiTools.informUser("Sorry, noch nicht Implementiert...",
                "Das Speichern des Graphen im .ged-Format ist noch nicht implementiert.");
        logger.log("save ged file");
    }

    /** Speichert den Graphen im .ged2-Format. */
    @Override
    public void saveGed2File() {
        // TODO : implement
        GuiTools.informUser("Sorry, noch nicht Implementiert...",
                "Das Speichern des Graphen im .ged2-Format ist noch nicht implementiert.");
        logger.log("save ged2 file");
    }

    /** Speichert den Graphen im .ged3-Format. */
    @Override
    public void saveGed3File() {
        String filename = gui.askUserToSaveAsGed3();
        if (filename.isEmpty()) {
            setStatus("Abbruch durch den Benutzer.");
            return;
        }

        if (!filename.endsWith(".ged3")) {
            filename = filename + ".ged3";
        }
        saveGed3File(filename);
        setStatus("Datei '" + filename + "' erfolgreich geladen.");
        logger.log("save ged3 file '" + filename + "'");
    }

    /**
     * Speichert den Graphen in der übergebenen Datei ab.
     *
     * @param file
     *            File-Objekt in dem der Graph gespeichert werden soll.
     */
    private void saveGed3File(String filename) {
        gui.startLongTimeProcess("Speichere Datei im GED3-Format...");

        Graph graph = gui.getGraph();
        String output = graph.saveToGed3Format();
        FileHelper.writeTextToFile(output, filename);
        // Wo lege ich den Dateinamen ab? Gar nicht, da altes Format

        gui.endLongTimeProcess();
    }

    /** Speichert den Graphen im .ged4-Format. */
    @Override
    public void saveGed4File() {
        if (filename.isEmpty()) {
            saveGed4FileAs();
        }
        else {
            saveGed4File(filename);
        }
    }

    /**
     * Speichert den Graphen in der übergebenen Datei ab.
     *
     * @param filename
     *            Name der Datei, unter der der Graph gespeichert werden soll.
     */
    private void saveGed4File(String filename) {
        gui.startLongTimeProcess("Speichere Datei im GED4-Format...");

        Graph graph = gui.getGraph();
        String output = graph.saveToGed4Format();
        FileHelper.writeTextToFile(output, filename);

        setStatus("Datei '" + filename + "' erfolgreich gespeichert.");
        gui.endLongTimeProcess();
        logger.log("save ged4 file '" + filename + "'");
    }

    /**
     * Speichert den Graphen (im .ged4-Format) unter einem anderen Namen, nach dem der Benutzer
     * gefragt wird.
     */
    @Override
    public void saveGed4FileAs() {
        String filename = gui.askUserToSaveAsGed4(this.filename);
        if (filename.isEmpty()) {
            setStatus("Abbruch durch den Benutzer.");
            return;
        }

        if (!filename.endsWith(".ged4")) {
            filename = filename + ".ged4";
        }

        saveGed4File(filename);
        this.filename = filename;
        gui.showFileName(filename);
        logger.log("save ged4 file as '" + filename + "'");
    }

    /** Einen Schritt in der Änderunghistorie zurück. */
    @Override
    public void undo() {
        if (history.hasPrevious()) {
            Graph lastGraph = history.getPrevious();
            showUndoOrRedoEditorState(lastGraph);
            logger.log("undo");
        }
    }

    /** Einen Schritt in der Änderunghistorie vorwärts. */
    @Override
    public void redo() {
        if (history.hasNext()) {
            Graph nextGraph = history.getNext();
            showUndoOrRedoEditorState(nextGraph);
            logger.log("redo");
        }
    }

    private void showUndoOrRedoEditorState(Graph graph) {
        checkEnableHistory();
        gui.setGraph(graph);
        repaint();
    }

    /** Überprüft das Anzeigen der undo- und redo-Schalter. */
    private void checkEnableHistory() {
        HistoryHelper.checkEnableHistory(history, gui);
    }

    /** Kein Update, nur repaint. */
    private void repaint() {
        gui.repaintGraphPanel();
    }

    /** Speichert die aktuelle Darstellung des Graphen im Panel als Bild ab. */
    @Override
    public void screenhot() {
        String filename = gui.askUserAboutNameForScreenshot();
        if (filename.isEmpty()) {
            setStatus("Abbruch durch den Benutzer.");
            return;
        }

        if (!filename.endsWith(".png")) {
            filename = filename + ".png";
        }

        try {
            ImageIO.write(gui.getSnapshot(), "png", new File(filename));
            logger.log("screenshot '" + filename + "'");
        }
        catch (IOException exception) {
            gui.error("Fehler beim Anfertigen eines Screenshots.", exception);
        }
    }

    /** Wählt die Defaultfarbe für die Ecken aus. */
    @Override
    public void chooseDefaultVertexColor() {
        GedColor color = gui.chooseDefaultVertexColor();
        gui.getGraph().getMeta().setDefaultVertexColor(color);
        update();
        logger.log("choosen default vertex color as " + color);
    }

    /** Wählt die Defaultfarbe für die Kanten aus. */
    @Override
    public void chooseDefaultEdgeColor() {
        GedColor color = gui.chooseDefaultEdgeColor();
        gui.getGraph().getMeta().setDefaultEdgeColor(color);
        update();
        logger.log("choosen default edge color as " + color);
    }

    /** Zeigt den Dialog zum Bearbeiten der Eckeneigenschaften an. */
    @Override
    public void showVertexPopupMenu(Vertex vertex, PixelPoint coords) {
        gui.showVertexPopupMenu(vertex, coords);
        logger.log("vertex dialog for vertex " + vertex + " at coord "
                + coords + " shown");
    }

    /** Zeigt den Dialog zum Bearbeiten der Kanteneigenschaften an. */
    @Override
    public void showEdgePopupMenu(Edge edge, PixelPoint coords) {
        gui.showEdgePopupMenu(edge, coords);
        logger.log("edge dialog for edge " + edge + " at coord " + coords + " shown");
    }

    /** Zeigt den Graphen in Textform an. */
    @Override
    public void showGraph() {
        String title = "Graph "; // + gui.getGraphPanel().getFileName();
        StringBuilder builder = new StringBuilder();
        Graph graph = gui.getGraph();
        builder.append(graph.toString());
        gui.showTextDialog(title, builder.toString());
        logger.log("graph as text shown");
    }

    /**
     * Bestimmt aus einer Menge von Kanten, welche davon gelöscht werden sollen
     *
     * @param edgesAtCoords
     *            Menge mit den Kanten, die in der Nähe einer bestimmten Bildschirmkoordinate
     *            verlaufen.
     * @param coords
     *            Koordinaten
     * @return Menge der Kanten, die wirklich gelöscht werden sollen.
     */
    @Override
    public Edges determineEdgesToDelete(Edges edgesAtCoords, PixelPoint coords) {
        return gui.determineEdgesToDelete(edgesAtCoords, coords);
    }

    /** Schaltet das Anzeigen des Gitters um. */
    @Override
    public void toggleMetaShowGrid() {
        Graph graph = gui.getGraph();
        MetaData meta = graph.getMeta();
        meta.setShowGrid(!meta.isShowGrid());
        gui.repaintGraphPanel();
        update();
        logger.log("toggle show grid to " + meta.isShowGrid());
    }

    /** Schaltet das Einrasten an den Gitterlinien ein oder aus. */
    @Override
    public void toggleMetaSnapInGrid() {
        Graph graph = gui.getGraph();
        MetaData meta = graph.getMeta();
        meta.setSnapInGrid(!meta.isSnapInGrid());
        gui.repaintGraphPanel();
        update();
        logger.log("toggle snap in grid to " + meta.isSnapInGrid());
    }

    /** Wählt den Defaultradius für Ecken. */
    @Override
    public void chooseDefaultVertexRadius() {
        Graph graph = gui.getGraph();
        MetaData meta = graph.getMeta();
        int radius = gui.askUserForSizeValue(meta.getDefaultRadius(), "Default-Radius");
        if (radius > 0) {
            meta.setDefaultRadius(radius);
            update();
        }
        logger.log("set default vertex radius to "
                + meta.getDefaultRadius());
    }

    /** Wählt die Standarddicke für Kanten. */
    @Override
    public void chooseDefaultEdgeWidth() {
        Graph graph = gui.getGraph();
        MetaData meta = graph.getMeta();
        int width = gui.askUserForSizeValue(meta.getDefaultEdgeWidth(), "Default-Kantendicke");
        if (width > 0) {
            meta.setDefaultEdgeWidth(width);
            update();
        }
        logger.log("set default edge width to "
                + meta.getDefaultEdgeWidth());
    }

    /** Wählt den Abstand der Gitterlinien. */
    @Override
    public void chooseGridDistance() {
        Graph graph = gui.getGraph();
        MetaData meta = graph.getMeta();
        int distance = gui.askUserForSizeValue(meta.getGridDistance(), "Gitterabstand");
        if (distance > 0) {
            meta.setGridDistance(distance);
            update();
        }
        logger.log("set grid distance to " + meta.getGridDistance());
    }

    /** Wählt die Gitterfarbe aus. */
    @Override
    public void chooseGridColor() {
        GedColor color = gui.chooseDefaultVertexColor();
        Graph graph = gui.getGraph();
        MetaData meta = graph.getMeta();
        meta.setGridColor(color);
        update();
        logger.log("set grid color to " + meta.getGridColor());
    }

    /** Wählt die Hintergrundfarbe aus. */
    @Override
    public void chooseBackgroundColor() {
        GedColor color = gui.chooseDefaultVertexColor();
        Graph graph = gui.getGraph();
        MetaData meta = graph.getMeta();
        meta.setBackgroundColor(color);
        gui.repaintGraphPanel();
        update();
        logger.log("set background color to " + meta.getBackgroundColor());
    }

    /** Wählt ein Hintergrundbild aus. */
    @Override
    public void chooseBackgroundPicture() {
        String filename = gui.askUserToOpenBackgroundPicture();
        if (filename.isEmpty()) {
            setStatus("Abbruch durch den Benutzer.");
            return;
        }

        if (filename.endsWith(".jpg")) {
            Graph graph = gui.getGraph();
            MetaData meta = graph.getMeta();
            meta.setBackgroundPicture(filename);
            gui.repaintGraphPanel();
            setStatus("Hintergrundbild '" + filename + "' erfolgreich geladen.");
            update();
            logger.log("set background picture to '" + filename + "'");
        }
        else {
            gui.error("Die Datei '" + filename + "' liegt nicht im .jpg-Format vor.");
        }
    }

    /** Entfernt das Hintergrundbild. */
    @Override
    public void removeBackgroundPicture() {
        Graph graph = gui.getGraph();
        MetaData meta = graph.getMeta();
        String oldPicture = meta.getBackgroundPicture();
        if (!oldPicture.isEmpty()) {
            meta.setBackgroundPicture("");
            gui.repaintGraphPanel();
            setStatus("Hintergrundbild '" + oldPicture + "' wurde entfernt.");
            logger.log("removed background picture '" + oldPicture + "'");
        }
    }

    /** Bewegt den Graphen um den Gitterabstand nach links. */
    @Override
    public void moveLeft() {
        Graph graph = gui.getGraph();
        graph.moveLeft();
        gui.repaintGraphPanel();
        update();
        logger.log("move graph left");
    }

    /** Bewegt den Graphen um den Gitterabstand nach rechts. */
    @Override
    public void moveRight() {
        Graph graph = gui.getGraph();
        graph.moveRight();
        gui.repaintGraphPanel();
        update();
        logger.log("move graph right");
    }

    /** Bewegt den Graphen um den Gitterabstand nach oben. */
    @Override
    public void moveUp() {
        Graph graph = gui.getGraph();
        graph.moveUp();
        gui.repaintGraphPanel();
        update();
        logger.log("move graph up");
    }

    /** Bewegt den Graphen um den Gitterabstand nach unten. */
    @Override
    public void moveDown() {
        Graph graph = gui.getGraph();
        graph.moveDown();
        gui.repaintGraphPanel();
        update();
        logger.log("move graph down");
    }

    /**
     * Zentriert den Graphen auf den Mittelpunkt der sichtbaren Fläche.
     *
     * @param minX
     *            x-Koordinate der linken oberen Ecke der Fläche.
     * @param minY
     *            y-Koordinate der linken oberen Ecke der Fläche.
     * @param maxX
     *            x-Koordinate der rechten unteren Ecke der Fläche.
     * @param maxY
     *            y-Koordinate der rechten unteren Ecke der Fläche.
     */
    @Override
    public void centerGraph(int minX, int minY, int maxX, int maxY) {
        Graph graph = gui.getGraph();
        graph.centerGraph(minX, minY, maxX, maxY);
        gui.repaintGraphPanel();
        update();
        logger.log("center graph");
    }

    /**
     * Zentriert den Graphen auf den Mittelpunkt der sichtbaren Fläche und vergrößert ihn so weit
     * er dann noch auf den Schirm passt.
     *
     * @param minX
     *            x-Koordinate der linken oberen Ecke der Fläche.
     * @param minY
     *            y-Koordinate der linken oberen Ecke der Fläche.
     * @param maxX
     *            x-Koordinate der rechten unteren Ecke der Fläche.
     * @param maxY
     *            y-Koordinate der rechten unteren Ecke der Fläche.
     */
    @Override
    public void maximizeGraph(int minX, int minY, int maxX, int maxY) {
        Graph graph = gui.getGraph();
        graph.maximizeGraph(minX, minY, maxX, maxY);
        gui.repaintGraphPanel();
        update();
        logger.log("maximize graph");
    }

    /**
     * Verkleinert den Graphen.
     *
     * @param minX
     *            x-Koordinate der linken oberen Ecke der Fläche.
     * @param minY
     *            y-Koordinate der linken oberen Ecke der Fläche.
     * @param maxX
     *            x-Koordinate der rechten unteren Ecke der Fläche.
     * @param maxY
     *            y-Koordinate der rechten unteren Ecke der Fläche.
     */
    @Override
    public void reduceGraph(int minX, int minY, int maxX, int maxY) {
        Graph graph = gui.getGraph();
        graph.reduceGraph(minX, minY, maxX, maxY);
        gui.repaintGraphPanel();
        update();
        logger.log("reduce graph");
    }

    /**
     * Vergrößert den Graphen.
     *
     * @param minX
     *            x-Koordinate der linken oberen Ecke der Fläche.
     * @param minY
     *            y-Koordinate der linken oberen Ecke der Fläche.
     * @param maxX
     *            x-Koordinate der rechten unteren Ecke der Fläche.
     * @param maxY
     *            y-Koordinate der rechten unteren Ecke der Fläche.
     */
    @Override
    public void enlargeGraph(int minX, int minY, int maxX, int maxY) {
        Graph graph = gui.getGraph();
        graph.enlargeGraph(minX, minY, maxX, maxY);
        gui.repaintGraphPanel();
        update();
        logger.log("enlarge graph");
    }

    /** Lässt alle Ecken des Graphen im Gitter einrasten. */
    @Override
    public void snapAllVerticesToGrid() {
        Graph graph = gui.getGraph();
        graph.snapAllVerticesToGrid();
        gui.repaintGraphPanel();
        update();
        logger.log("snap all vertices to grid");
    }

    /** Zeigt die Größe des GraphPanels an. */
    @Override
    public void showGraphPanelSize(int width, int height) {
        gui.showGraphPanelSize(width, height);
    }

    /** Setzt eine Ecke in den Mittelpunkt der Bildfläche. */
    @Override
    public void insertCenter() {
        PixelPoint center = createCenter();
        Vertex vertex = new Vertex();
        vertex.setCoordinates(center);
        Graph graph = gui.getGraph();
        graph.addVertex(vertex);
        gui.repaintGraphPanel();
        update();
        logger.log("center inserted");
    }

    /** Ermittelt den Mittelpunkt des GraphenPanels. */
    private PixelPoint createCenter() {
        GraphPanel graphPanel = gui.getGraphPanel();
        int centerX = graphPanel.getWidth() / 2;
        int centerY = graphPanel.getHeight() / 2;
        PixelPoint center = new PixelPoint(centerX, centerY);
        return center;
    }

    /** Fügt einen Kreis um den Mittelpunkt der aktuellen Bildfläche ein. */
    @Override
    public void insertCircle(int numberOfVertices) {
        List<Vertex> vertexList = createCircleVertices(numberOfVertices);
        Graph graph = gui.getGraph();
        for (int i = 0; i < vertexList.size(); ++i) {
            Vertex from = vertexList.get(i);
            Vertex to;
            if (i == vertexList.size() - 1) {
                to = vertexList.get(0);
            }
            else {
                to = vertexList.get(i + 1);
            }
            graph.addEdge(from, to);
        }
        update();
        gui.repaintGraphPanel();
        logger.log("circle with " + numberOfVertices
                + " vertices inserted");
    }

    /** Erzeugt die Ecken eines Kreises und fügt sie zum Graphen hinzu. */
    private List<Vertex> createCircleVertices(int numberOfVertices) {
        /* Radius bestimmen: */
        PixelPoint center = createCenter();
        int radius = center.getX();
        if (center.getY() < radius) {
            radius = center.getY();
        }
        radius -= 20;

        List<Vertex> vertexList = new ArrayList<>();

        Graph graph = gui.getGraph();
        for (int n = 1; n <= numberOfVertices; ++n) {
            double phi = (n - 1.0) * (2.0 * Math.PI) / numberOfVertices;
            double x = radius * Math.cos(phi);
            double y = radius * Math.sin(phi);
            x += center.getX();
            y += center.getY();
            PixelPoint coord = new PixelPoint(x, y);
            Vertex vertex = new Vertex();
            vertex.setCoordinates(coord);
            graph.addVertex(vertex);
            vertexList.add(vertex);
        }

        return vertexList;
    }

    /** Fügt einen vollständigen Graphen um den Mittelpunkt der aktuellen Bildfläche ein. */
    @Override
    public void insertCompleteGraph(int numberOfVertices) {
        List<Vertex> vertexList = createCircleVertices(numberOfVertices);
        Graph graph = gui.getGraph();
        for (int i = 0; i < vertexList.size(); ++i) {
            for (int j = 0; j < vertexList.size(); ++j) {
                if (i != j) {
                    Vertex from = vertexList.get(i);
                    Vertex to = vertexList.get(j);
                    graph.addEdge(from, to);
                }
            }
        }
        update();
        gui.repaintGraphPanel();
        logger.log("complete graph with " + numberOfVertices
                + " vertices inserted");
    }

    /** Getter für die Programmversion. */
    @Override
    public Version getVersion() {
        return version;
    }

    /** Zeigt die Webseite an. */
    @Override
    public void webpage() {
        if (Desktop.isDesktopSupported()) {
            Desktop desktop = Desktop.getDesktop();
            if (desktop.isSupported(Desktop.Action.BROWSE)) {
                URI uri;
                try {
                    uri = new URI("http://www.duehl.de/christian/java/ged/ged_java.html");
                    desktop.browse(uri);
                }
                catch (URISyntaxException | IOException exception) {
                    gui.error("Es gab ein Problem beim Öffnen der Webseite.", exception);
                }
            }
        }
    }

    /**
     * Setzt die Größe der Canvas.
     *
     * @param width
     *            neue Breite
     * @param height
     *            neue Höhe
     */
    @Override
    public void setViewSize(int width, int height) {
        gui.setViewSize(width, height);
        logger.log("set view size to " + width + " x " + height);
    }

    /** Gibt den Graphen zurück, der angezeigt wird. */
    @Override
    public Graph getGraph() {
        return gui.getGraph();
    }

    /** Gibt den Logger zurück. */
    @Override
    public Logger getLogger() {
        return logger;
    }

    /** Getter für das Verzeichnis ~/.ged für Einstellungen und Graphendateien. */
    @Override
    public String getDirectory() {
        return directory;
    }

    /** Setzt die Farben aller Ecken auf Default. */
    @Override
    public void setAllVertexColorsToDefault() {
        Graph graph = gui.getGraph();
        graph.setAllVertexColorsToDefault();
        update();
    }

    /** Setzt die Radien aller Ecken auf Default. */
    @Override
    public void setAllVertexRadiusToDefault() {
        Graph graph = gui.getGraph();
        graph.setAllVertexRadiusToDefault();
        update();
    }

    /** Setzt die Farben aller Kanten auf Default. */
    @Override
    public void setAllEdgeColorsToDefault() {
        Graph graph = gui.getGraph();
        graph.setAllEdgeColorsToDefault();
        update();
    }

    /** Setzt die Dicken aller Kanten auf Default. */
    @Override
    public void setAllEdgeWidthsToDefault() {
        Graph graph = gui.getGraph();
        graph.setAllEdgeWidthsToDefault();
        update();
    }

    /** Schaltet den Graphen zwischen gerichtet und ungerichtet um. */
    @Override
    public void switchDirectedGraph(boolean directed) {
        Graph graph = gui.getGraph();
        graph.setDirected(directed);
        gui.switchDirectedGraph(directed);
        update();
    }

}
