package de.duehl.math.graph.ged.ui.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;

import javax.swing.JButton;
import javax.swing.JPanel;

import de.duehl.basics.text.NumberString;
import de.duehl.math.graph.ged.graph.GedColor;
import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.graph.MetaData;
import de.duehl.math.graph.ged.graph.vertex.Vertex;
import de.duehl.math.graph.ged.pixel.PixelPoint;
import de.duehl.math.graph.ged.ui.GedGuiTools;
import de.duehl.math.graph.ged.ui.GuiElements;
import de.duehl.math.graph.ged.ui.elements.SubGraphButton;
import de.duehl.math.graph.ged.ui.elements.SwitchableValueDistanceAndColorChooser;
import de.duehl.math.graph.ged.ui.elements.ValueDistanceAndColorChooser;
import de.duehl.math.graph.ged.ui.elements.color.DefaultOrOwnColorChooser;
import de.duehl.math.graph.ged.ui.elements.radiusandwidth.RadiusOrWidthChooser;
import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.dialogs.base.ModalDialogBase;
import de.duehl.swing.ui.layout.FixedCenter;

/**
 * Diese Klasse stellt den Dialog für die Bearbeitung der Eigenschaften einer Ecke dar.
 *
 * @version 1.01     2019-07-02
 * @author Christian Dühl
 */

public class VertexDialog extends ModalDialogBase {

    private static final int SUB_GRAPH_SIZE = 150;

    private static final Color NOT_SHOWN_COLOR = Color.MAGENTA; // Wird nicht angezeigt

    /** Graph, der dargestellt wird. */
    private Graph graph;

    /** Metadaten des Graphen, der dargestellt wird, zur Bequemlichkeit. */
    private final MetaData meta;

    /** Kopie der Ecke, deren Daten der Dialog anzeigt und ändern lässt. */
    private Vertex vertex;

    /** Ecke, deren Daten der Dialog anzeigt und ändern lässt. */
    private Vertex originalVertex;

    /** Koordinaten, an denen der Dialog angezeigt werden soll. */
    private PixelPoint coords;

    /** Enthält die grafischen Elemente. */
    private GuiElements elements;

    /* grafische Elemente: */
    private SubGraphButton subGraphButton;
    private DefaultOrOwnColorChooser colorChooser;
    private RadiusOrWidthChooser radiusChooser;
    private ValueDistanceAndColorChooser labelChooser;
    private SwitchableValueDistanceAndColorChooser weightChooser;

    /**
     * Konstruktor.
     *
     * @param vertex
     *            Ecke, deren Daten der Dialog anzeigt und ändern lässt.
     * @param coords
     *            Koordinaten, an denen der Dialog angezeigt werden soll.
     * @param elements
     *            Enthält die grafischen Elemente.
     */
    public VertexDialog(Graph graph, Vertex vertex, PixelPoint coords, GuiElements elements) {
        super(elements.getFrameLocation(), elements.getProgramImage(),
                "Eigenschaften der Ecke mit Index " + vertex.getIndex());
        addEscapeBehaviour();

        this.graph = graph;
        meta = graph.getMeta();
        this.originalVertex = vertex;
        this.coords = coords;
        this.elements = elements;

        Graph subGraph = graph.createVertexWithNeighboursSubGraph(vertex, SUB_GRAPH_SIZE);
        subGraphButton = new SubGraphButton(subGraph, SUB_GRAPH_SIZE);
        this.vertex = subGraph.getVertex(0);

        fillDialog();
    }

    @Override
    protected void populateDialog() {
        add(createMainPart(), BorderLayout.CENTER);
        add(createButtonPart(), BorderLayout.SOUTH);

        setLocation();
    }

    private void setLocation() {
        MouseClickLocationSetter setter = new MouseClickLocationSetter(this, coords, elements);
        setter.setLocation();
    }

    private Component createMainPart() {
        JPanel panel = new JPanel();
        panel.setLayout(GedGuiTools.createLeftAndRightMainPartLayout());

        panel.add(createVertexColorPanel());
        panel.add(createRadiusPanel());
        panel.add(createLabelPanel());
        panel.add(createWeightPanel());
        panel.add(createSubGraphPanel());
        panel.add(createDeleteVertexButton());

        return panel;
    }

    private Component createVertexColorPanel() {
        colorChooser = new DefaultOrOwnColorChooser(
                vertex.hasDefaultColor(),
                "Default-Farbe verwenden",
                "Eckenfarbe: ",
                "Eckenfarbe für die Ecke mit Index " + vertex.getIndex() + " wählen:",
                vertex.hasDefaultColor() ? NOT_SHOWN_COLOR
                                         : vertex.getColor().toSwingColor(),
                meta.getDefaultVertexColor(),
                gedColor -> userSetsVertexColor(gedColor),
                elements.getFrame());
        JPanel panel = colorChooser.createPanel();
        GuiTools.createTitle("Eckenfarbe", panel);

        return panel;
    }

    private void userSetsVertexColor(GedColor gedColor) {
        vertex.setColor(gedColor);
        if (!GedColor.isDefaultColor(gedColor)) {
            labelChooser.setDefaultButtonColor(gedColor);
            weightChooser.setDefaultButtonColor(gedColor);
        }
        refresh();
    }

    private Component createRadiusPanel() {
        radiusChooser = new RadiusOrWidthChooser(vertex.hasDefaultRadius(),
                "Default-Radius verwenden",
                Vertex.USE_DEFAULT_RADIUS,
                meta.getDefaultRadius(),
                "Radius: ",
                radius -> radiusChanged(radius),
                coords,
                elements);
        radiusChooser.setValue(vertex.getRadius());
        radiusChooser.addChangeAction(() -> radiusChanged());
        JPanel panel = radiusChooser.getPanel();
        GuiTools.createTitle("Radius der Ecke", panel);
        return panel;
    }

    private void radiusChanged(int radius) {
        vertex.setRadius(radius);
        refresh();
    }

    private void radiusChanged() {
        if (radiusChooser.isDefaultSelected()) {
            vertex.setRadiusToDefault();
        }
        else {
            String radiusText = radiusChooser.getValueText();
            if (NumberString.isDigitSequence(radiusText)) {
                int radius = NumberString.parseIntIgnore(radiusText, -1);
                if (radius > 0) {
                    vertex.setRadius(radius);
                }
            }
        }
        refresh();
    }

    private JPanel createLabelPanel() {
        labelChooser = new ValueDistanceAndColorChooser(
                "Eckenbeschriftung: ",
                vertex.hasDefaultLabelColor(),
                "X-Abstand der Eckenbeschriftung: ",
                "Y-Abstand der Eckenbeschriftung: ",
                "Default-Farbe für Beschriftung verwenden",
                "Farbe der Beschriftung: ",
                "Farbe der Beschriftung für die Ecke mit Index " + vertex.getIndex() + " wählen:",
                vertex.hasDefaultLabelColor() ? NOT_SHOWN_COLOR
                                              : vertex.getLabelColor().toSwingColor(),
                vertex.hasDefaultColor() ? meta.getDefaultVertexColor()
                        : vertex.getColor(),
                gedColor -> userSetsLabelColor(gedColor),
                elements.getFrame());
        labelChooser.setStartValue(vertex.getLabel());
        labelChooser.setXDistance(vertex.getLabelDistanceX());
        labelChooser.setYDistance(vertex.getLabelDistanceY());
        labelChooser.addChangeValueAction(() -> labelValueChanged());
        labelChooser.addChangeXDistanceAction(() -> labelXDistanceChanged());
        labelChooser.addChangeYDistanceAction(() -> labelYDistanceChanged());
        JPanel panel = labelChooser.getPanel();
        GuiTools.createTitle("Beschriftung", panel);

        return panel;
    }

    private void userSetsLabelColor(GedColor gedColor) {
        vertex.setLabelColor(gedColor);
        refresh();
    }

    private void labelValueChanged() {
        vertex.setLabel(labelChooser.getValue());
        refresh();
    }

    private void labelXDistanceChanged() {
        int labelX = NumberString.parseIntIgnore(labelChooser.getXDistanceFieldText(),
                Integer.MIN_VALUE);
        if (labelX > Integer.MIN_VALUE) {
            vertex.setLabelDistanceX(labelX);
        }
        refresh();
    }

    private void labelYDistanceChanged() {
        int labelY = NumberString.parseIntIgnore(labelChooser.getYDistanceFieldText(),
                Integer.MIN_VALUE);
        if (labelY > Integer.MIN_VALUE) {
            vertex.setLabelDistanceY(labelY);
        }
        refresh();
    }

    private JPanel createWeightPanel() {
        weightChooser = new SwitchableValueDistanceAndColorChooser(
                vertex.hasWeight(),
                "Gewicht verwenden",
                "Gewicht der Ecke: ",
                vertex.hasDefaultWeightColor(),
                "X-Abstand des Gewichts: ",
                "Y-Abstand des Gewichts: ",
                "Default-Farbe für das Gewicht verwenden",
                "Farbe des Gewichts: ",
                "Farbe das Gewicht der Ecke mit Index " + vertex.getIndex() + " wählen:",
                vertex.hasDefaultWeightColor() ? NOT_SHOWN_COLOR
                                               : vertex.getWeightColor().toSwingColor(),
                vertex.hasDefaultColor() ? meta.getDefaultVertexColor()
                                         : vertex.getColor(),
                gedColor -> userSetsWeightColor(gedColor),
                elements.getFrame());
        weightChooser.setStartValue(Double.toString(vertex.getWeight()));
        weightChooser.setXDistance(vertex.getWeightDistanceX());
        weightChooser.setYDistance(vertex.getWeightDistanceY());
        weightChooser.addChangeValueAction(() -> weightValueChanged());
        weightChooser.addChangeXDistanceAction(() -> weightXDistanceChanged());
        weightChooser.addChangeYDistanceAction(() -> weightYDistanceChanged());
        JPanel panel = weightChooser.getPanel();
        GuiTools.createTitle("Gewicht der Ecke", panel);

        return panel;
    }

    private void userSetsWeightColor(GedColor gedColor) {
        vertex.setWeightColor(gedColor);
        refresh();
    }

    private void weightValueChanged() {
        String value = weightChooser.getValue();
        double weight = NumberString.parseDoubleIgnore(value, Double.MIN_VALUE);
        if (weight > Double.MIN_VALUE) {
            vertex.setWeight(weight);
        }
        refresh();
    }

    private void weightXDistanceChanged() {
        int x = NumberString.parseIntIgnore(weightChooser.getXDistanceFieldText(),
                Integer.MIN_VALUE);
        if (x > Integer.MIN_VALUE) {
            vertex.setWeightDistanceX(x);
        }
        refresh();
    }

    private void weightYDistanceChanged() {
        int y = NumberString.parseIntIgnore(weightChooser.getYDistanceFieldText(),
                Integer.MIN_VALUE);
        if (y > Integer.MIN_VALUE) {
            vertex.setWeightDistanceY(y);
        }
        refresh();
    }

    private Component createSubGraphPanel() {
        FixedCenter fixedCenter = new FixedCenter(subGraphButton);
        return fixedCenter.createPanel();
    }
    private Component createDeleteVertexButton() {
        JButton button = new JButton("Ecke löschen");
        button.addActionListener(e -> deleteVertex());
        return button;
    }

    private void deleteVertex() {
        graph.removeVertex(originalVertex);
        closeDialog();
    }

    private Component createButtonPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(createQuitButton(), BorderLayout.WEST);
        panel.add(createOkButton(), BorderLayout.EAST);

        return panel;
    }

    private Component createQuitButton() {
        JButton button = new JButton("Abbrechen");
        button.addActionListener(e -> closeDialog());
        return button;
    }

    private Component createOkButton() {
        JButton button = new JButton("OK");
        button.addActionListener(e -> pressedOkButton());
        return button;
    }


    /** Reagiert auf den Druck auf den OK-Button. */
    private void pressedOkButton() {
        copyValues();
        closeDialog();
    }

    private void copyValues() {
        int index = originalVertex.getIndex();
        int x = originalVertex.getX();
        int y = originalVertex.getY();

        originalVertex.copyValuesFrom(vertex);

        originalVertex.setIndex(index);
        originalVertex.setX(x);
        originalVertex.setY(y);
    }

}
