package de.duehl.math.graph.ged.graph;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.duehl.basics.text.Text;
import de.duehl.basics.text.html.generation.HtmlBuilder;
import de.duehl.math.geometry.Line;
import de.duehl.math.graph.ged.graph.edge.AdjacenceEntry;
import de.duehl.math.graph.ged.graph.edge.AdjacenceLists;
import de.duehl.math.graph.ged.graph.edge.AdjacenceList;
import de.duehl.math.graph.ged.graph.io.Ged1FormatLoader;
import de.duehl.math.graph.ged.graph.io.Ged1FormatSaver;
import de.duehl.math.graph.ged.graph.io.Ged2FormatLoader;
import de.duehl.math.graph.ged.graph.io.Ged2FormatSaver;
import de.duehl.math.graph.ged.graph.io.Ged3FormatLoader;
import de.duehl.math.graph.ged.graph.io.Ged3FormatSaver;
import de.duehl.math.graph.ged.graph.io.Ged4FormatLoader;
import de.duehl.math.graph.ged.graph.io.Ged4FormatSaver;
import de.duehl.math.graph.ged.graph.subGraph.EdgeSubGraphCreator;
import de.duehl.math.graph.ged.graph.subGraph.VertexWithItsNeigboursSubGraphCreator;
import de.duehl.math.graph.ged.graph.vertex.Vertex;
import de.duehl.math.graph.ged.graph.vertex.Vertices;
import de.duehl.math.graph.ged.pixel.PixelPoint;

/**
 * Diese Klasse stellt einen Graphen dar.
 *
 * @version 1.01     2019-05-27
 * @author Christian Dühl
 */

public class Graph {

    private final static String LINE_SEPERATOR = Text.LINE_BREAK;

    /** Eckenmenge des Graphen. */
    private final Vertices vertices;

    /** Kantenmenge des Graphen in Form von Adjazenzlisten. */
    private final AdjacenceLists adjacenceLists;

    /** Enthält weitere Informationen zum Graphen. */
    private MetaData meta;
    // TODO : final machen, kopieren von Werten in MetaData!

    /** Konstruktor */
    public Graph() {
        vertices = new Vertices();
        adjacenceLists = new AdjacenceLists();
        meta = new MetaData();
    }

    /**
     * Erzeugt eine Kopie des übergebenen Graphen.
     *
     * @param graph
     *            zu kopierender Graph
     */
    public Graph(Graph graph) {
        this();
        synchronized (graph) {
            for (Vertex vertex : graph.vertices) {
                Vertex copyVertex = new Vertex(vertex);
                vertices.add(copyVertex);
            }
            for (AdjacenceList list : graph.adjacenceLists) {
                AdjacenceList myList = new AdjacenceList();
                adjacenceLists.add(myList);
                for (AdjacenceEntry entry : list) {
                    AdjacenceEntry copyEntry = new AdjacenceEntry(entry);
                    myList.add(copyEntry);
                }
            }
            meta = new MetaData(graph.meta);
        }
    }

    /**
     * Gibt die Anzahl der Ecken zurück.
     *
     * @return Anzahl der Ecken.
     */
    public int getNumberOfVertices() {
        return vertices.size();
    }

    /**
     * Überprüft, ob der Graph in sich noch valide ist.
     *
     * @return Wahrheitswert
     */
     boolean validate() { // nicht public
        /*
         * Überprüfen, ob die Indices der Ecken auch alle zu den Indices der
         * Liste passen:
         */
        for (int index = 0; index < vertices.size(); ++index) {
            Vertex vertex = vertices.get(index);
            int vertexIndex = vertex.getIndex();
            if (index != vertexIndex) {
                return false;
            }
        }

        /*
         * Stimmt die Größe der Eckenliste mit der Größe der Adjazenzlistenliste
         * überein?
         */
        if (vertices.size() != adjacenceLists.size()) {
            return false;
        }

        return true;
    }

    /**
     * Fügt eine Ecke hinzu.
     *
     * @param vertex
     *            Neue Ecke.
     */
    synchronized
    public void addVertex(Vertex vertex) {
        /*
         * Das gleiche Objekt darf nicht mehrfach in der Liste sein, da die
         * Indices sonst bei allen davon gleich wären:
         */
        if (vertices.contains(vertex)) {
            throw new IllegalArgumentException("Die Ecke " + vertex + " wurde schon hinzugefügt!");
        }

        /* Richtigen Index in der Ecke setzen: */
        int nextIndex = vertices.size();
        vertex.setIndex(nextIndex);

        /* Ecke hinzufügen: */
        vertices.add(vertex);

        /* Leere Liste der Nachbarn für diese Ecke hinzufügen: */
        AdjacenceList list = new AdjacenceList();
        adjacenceLists.add(list);

        /* Ecke im Gitter einrasten lassen, falls gewünscht: */
        if (meta.isSnapInGrid()) {
            snapVertexToGrid(vertex);
        }
    }

    /** Rastet die Ecke im Gitter ein. */
    synchronized
    public void snapVertexToGrid(Vertex vertex) {
        double distance = meta.getGridDistance();
        PixelPoint coord = vertex.getCoordinates();

        /* x */
        double x = coord.getX();
        double xFactor = x / distance;
        double xBefore = Math.floor(xFactor) * distance;
        double xBehind = Math.ceil(xFactor) * distance;
        if (x-xBefore < xBehind-x) {
            x = xBefore;
        }
        else {
            x = xBehind;
        }
        coord.setX(x);

        /* y */
        double y = coord.getY();
        double yFactor = y / distance;
        double yBefore = Math.floor(yFactor) * distance;
        double yBehind = Math.ceil(yFactor) * distance;
        if (y - yBefore < yBehind - y) {
            y = yBefore;
        }
        else {
            y = yBehind;
        }
        coord.setY(y);
    }

    /**
     * Kante zum Graphen hinzufügen.
     *
     * @param vertex
     *            Startecke.
     * @param entry
     *            Adjazenz-Objekt.
     */
    public void addEdge(Vertex vertex, AdjacenceEntry entry) {
        addEdge(vertex.getIndex(), entry);
    }

    /**
     * Kante zum Graphen hinzufügen.
     *
     * @param from
     *            Index der Start-Ecke (0 bis n-1).
     * @param entry
     *            Adjazenz-Objekt.
     */
    synchronized
    public void addEdge(int from, AdjacenceEntry entry) {
        /* Indices from und to überprüfen: */
        int to = entry.getNeighbourIndex();
        int n = getNumberOfVertices();
        if (from < 0 || from >= n) {
            throw new IllegalArgumentException("Ecke mit Start-Index " + from
                    + " nicht vorhanden.");
        }
        if (to < 0 || to >= n) {
            throw new IllegalArgumentException("Ecke mit End-Index " + to
                    + " nicht vorhanden.");
        }
        if (to == from) {
            throw new IllegalArgumentException("Keine Schleifen!");
        }

        /* Richtige Liste auswählen: */
        AdjacenceList list = adjacenceLists.get(from);

        /* AdjaceceEntry an diese Liste anhängen: */
        list.add(entry);
    }


    /**
     * Kante zum Graphen hinzufügen.
     *
     * @param from
     *            Index der Start-Ecke (0 bis n-1).
     * @param to
     *            Index der Ziel-Ecke (0 bis n-1).
     */
    public void addEdge(int from, int to) {
        addEdge(from, new AdjacenceEntry(to));
    }

    /**
     * Kante zum Graphen hinzufügen.
     *
     * @param vertexFrom
     *            Startecke der Kante.
     * @param vertexTo
     *            Endecke der Kante.
     */
    public void addEdge(Vertex vertexFrom, Vertex vertexTo) {
        int from = vertexFrom.getIndex();
        int to = vertexTo.getIndex();
        addEdge(from, to);
    }

    /**
     * Entfernt eine Kante aus dem Graphen.
     *
     * @param from
     *            Index der Start-Ecke (0 bis n-1).
     * @param to
     *            Index der Ziel-Ecke (0 bis n-1).
     */
    synchronized
    public void removeEdge(int from, int to) {
        /* Indices from und to überprüfen: */
        int n = getNumberOfVertices();
        if (from < 0 || from >= n) {
            throw new IllegalArgumentException("Ecke mit Start-Index " + from
                    + " nicht vorhanden.");
        }
        if (to < 0 || to >= n) {
            throw new IllegalArgumentException("Ecke mit End-Index " + to
                    + " nicht vorhanden.");
        }
        if (to == from) {
            throw new IllegalArgumentException("Keine Schleifen!");
        }

        /* Richtige Liste auswählen: */
        AdjacenceList list = adjacenceLists.get(from);

        /* Richtiges Element in dieser Liste finden und entfernen: */
        boolean removed = false;
        for (int index = 0; index < list.size(); ++index) {
            AdjacenceEntry entry = list.get(index);
            int neighbourIndex = entry.getNeighbourIndex();
            if (to == neighbourIndex) {
                list.remove(index);
                removed = true;
                break;
            }
        }
        if (!removed) {
            throw new RuntimeException("Konnte in den Adjazenzlisten die Kante von Index " + from
                    + " nach Index " + to + " nicht löschen.");
        }
    }

    /**
     * Entfernt eine Kante aus dem Graphen.
     *
     * @param vertexFrom
     *            Startecke der Kante.
     * @param vertexTo
     *            Endecke der Kante.
     */
    public void removeEdge(Vertex vertexFrom, Vertex vertexTo) {
        int from = vertexFrom.getIndex();
        int to = vertexTo.getIndex();
        removeEdge(from, to);
    }

    /**
     * Gibt die Anzahl der Kanten des Graphen zurück.
     *
     * @return Anzahl der (gerichteten) Kanten.
     */
    public int getNumberOfEdges() {
        return adjacenceLists.getNumberOfEdges();
    }

    /** Stringrepräsentation zurückgeben */
    @Override
    public String toString() {
        HtmlBuilder builder = new HtmlBuilder()
                .appendLn("Graph:")
                .increaseIndentationLevel()
                .appendLn("Anzahl Ecken : " + getNumberOfVertices())
                .appendLn("Anzahl Kanten: " + getNumberOfEdges())

                .appendLineBreak()
                .appendLn("Ecken:")
                .increaseIndentationLevel();

        for (Vertex vertex : vertices) {
            builder.appendMultipleLines(vertex.toString());
        }

        builder.appendLineBreak()
                .decreaseIndentationLevel()
                .appendLn("Kanten:")
                .increaseIndentationLevel();

        for (int index = 0; index < adjacenceLists.size(); ++index) {
            builder.appendLn("Kanten von der Ecke mit Index " + index + ":")
                    .increaseIndentationLevel();
            for (AdjacenceEntry entry : adjacenceLists.get(index)) {
                int targetIndex = entry.getNeighbourIndex();
                builder.appendMultipleLines("zur Ecke mit Index " + targetIndex
                        + ": " + LINE_SEPERATOR + entry);
            }
            builder.decreaseIndentationLevel();
        }

        builder.appendLineBreak()
                .decreaseIndentationLevel()
                .appendMultipleLines(meta.toString());

        return builder.toString();
    }

    /**
     * Entfernt die Ecke mit dem gegebenen Index.
     *
     * @param index
     *            Index der zu entfernenden Ecke (0 bis n-1).
     */
    synchronized
    public void removeVertex(int index) {
        /* Index überprüfen: */
        if (index < 0 || index >= getNumberOfVertices()) {
            throw new IllegalArgumentException("Ecke mit Index " + index
                    + " nicht vorhanden.");
        }

        /* Alle Kanten, die von dieser Ecke aus starten entfernen: */
        adjacenceLists.get(index).clear(); // nicht unbedingt nötig

        /* Alle Kanten, die zu dieser Ecke gehen entfernen: */
        for (AdjacenceList list : adjacenceLists) {
            /* Alle zu löschenden Indices merken: */
            List<Integer> removeList = new ArrayList<Integer>();
            for (int listIndex=0; listIndex<list.size(); ++listIndex) {
                AdjacenceEntry entry = list.get(listIndex);
                if (index == entry.getNeighbourIndex()) {
                    removeList.add(listIndex);
                }
            }
            /* Andersherum sortieren (groß nach klein): */
            Collections.reverse(removeList);
            /* Elemente mit diesen Indices entfernen: */
            for (int removeIndex : removeList) {
                list.remove(removeIndex);
            }
        }


        /* Alle Ecken mit größerem Index reduzieren: */
        for (int upperIndex=index+1; upperIndex<vertices.size(); ++ upperIndex) {
            Vertex vertex = vertices.get(upperIndex);
            int vertexIndex = vertex.getIndex();
            if (vertexIndex != upperIndex) {
                throw new RuntimeException("Interner Fehler beim Entfernen der " +
                        "Ecke mit Index " + index + " (1).");
            }
            --vertexIndex;
            vertex.setIndex(vertexIndex);
        }

        /* Index der Ecke auf -1 setzen: */
        vertices.get(index).setIndex(-1);

        /* Ecke selbst entfernen: */
        vertices.remove(index);

        /* Alle Nachbarn mit größerem Index reduzieren: */
        for (AdjacenceList list : adjacenceLists) {
            for (AdjacenceEntry entry : list) {
                int neighbourIndex = entry.getNeighbourIndex();
                if (neighbourIndex > index) {
                    --neighbourIndex;
                    entry.setNeighbourIndex(neighbourIndex);
                }
            }
        }

        /* AdjazenzListe zu dieser Ecke entfernen: */
        adjacenceLists.remove(index);

        /* Graph überprüfen: */
        if (!validate()) {
            throw new RuntimeException("Interner Fehler beim Entfernen der " +
                    "Ecke mit Index " + index + " (2).");
        }
    }

    /**
     * Entfernt die Ecke.
     *
     * @param vertex
     *            Zu entfernende Ecke.
     */
    public void removeVertex(Vertex vertex) {
        removeVertex(vertex.getIndex());
    }

    /**
     * Verschmilzt die Ecke mit dem gegebenen Index.
     *
     * @param index
     *            Index der zu verschmelzenden Ecke (0 bis n-1).
     */
    synchronized
    public void mergeVertex(int index) {
        /*
         * Neue Kanten einfügen:
         */

        /* 1. Nachbarn, von denen Kanten zu dieser Ecke führen, ermitteln: */
        List<Vertex> neighboursBackwards = getNeighboursBackwardsAsVertices(index);

        /* 2. Nachbarn, zu denen Kanten von dieser Ecke führen, ermitteln: */
        List<Vertex> neighboursForewards = getNeighboursAsVertices(index);

        /*
         * 3. Von allen Nachbarn vom Typ 1 zu allen Nachbarn vom Typ 2 Kanten
         * ziehen, falls keine da sind und sie nicht identisch sind.
         */
        for (Vertex backwardsNeighbour : neighboursBackwards) {
            for (Vertex forewardsNeighbour : neighboursForewards) {
                if (forewardsNeighbour.equals(backwardsNeighbour)) {
                    continue;
                }
                if (existsEdge(backwardsNeighbour, forewardsNeighbour)) {
                    continue;
                }
                addEdge(backwardsNeighbour, forewardsNeighbour);
            }
        }

        /* Ecke entfernen: */
        removeVertex(index);
    }

    /**
     * Prüft, ob eine Kante von der startVertex zur endVertex existiert.
     *
     * @param startVertex
     *            Startecke der Kante.
     * @param endVertex
     *            Zielecke der Kante.
     */
    public boolean existsEdge(Vertex startVertex, Vertex endVertex) {
        int startIndex = startVertex.getIndex();
        int endIndex = endVertex.getIndex();
        return existsEdge(startIndex, endIndex);
    }

    /**
     * Prüft, ob eine Kante von der startVertex zur endVertex existiert.
     *
     * @param startIndex
     *            Index der Startecke der Kante.
     * @param endEndex
     *            Index der Zielecke der Kante.
     */
    public boolean existsEdge(int startIndex, int endIndex) {
        AdjacenceList list = adjacenceLists.get(startIndex);
        for (AdjacenceEntry entry : list) {
            int neighbourIndex = entry.getNeighbourIndex();
            if (neighbourIndex == endIndex) {
                return true;
            }
        }
        return false;
    }

    /**
     * Verschmilzt die Ecke mit dem gegebenen Index.
     *
     * @param vertex
     *            Zu verschmelzende Ecke.
     */
    public void mergeVertex(Vertex vertex) {
        mergeVertex(vertex.getIndex());
    }

    /** Liefert die Liste mit den Ecken zurück. */
    public Vertices getVertices() {
        return vertices;
    }
    // TODO Verwendung diese Methode vermeiden, alles was anderswo damit gemacht wird, in Graph
    // einbauen.

    /**
     * Liefert die Ecke zum gegebenen Index zurück.
     *
     * @param index
     *            Index der gesuchten Ecke.
     * @return Gesuchte Ecke.
     */
    public Vertex getVertex(int index) {
        return vertices.get(index);
    }

    /**
     * Gibt die Liste mit den Nachbarecken der übergebenen Ecke zurück.
     *
     * @param vertex
     *            Ausgangsecke
     * @return Liste mit den Nachbarn.
     */
    public AdjacenceList getNeighbours(Vertex vertex) {
        return adjacenceLists.get(vertex.getIndex());
    }

    /**
     * Gibt die Liste mit den Nachbarecken der Ecke mit dem übergebenen Index
     * zurück.
     *
     * @param index
     *            Index der Ecke.
     * @return Liste mit den Nachbarn.
     */
    public AdjacenceList getNeighbours(int index) {
        return adjacenceLists.get(index);
    }

    /**
     * Gibt die Nachbarn als Liste von Ecken zurück.
     *
     * @param index
     *            Index der Ecke.
     * @return Liste mit den Nachbarn.
     */
    public List<Vertex> getNeighboursAsVertices(int index) {
        List<Vertex> neighbours = new ArrayList<>();
        AdjacenceList list = adjacenceLists.get(index);
        for (AdjacenceEntry entry : list) {
            int neighbourIndex = entry.getNeighbourIndex();
            Vertex neighbour = vertices.get(neighbourIndex);
            neighbours.add(neighbour);
        }
        return neighbours;
    }

    /**
     * Gibt die Nachbarn als Liste von Ecken zurück.
     *
     * @param vertex
     *            Ausgangsecke
     * @return Liste mit den Nachbarn.
     */
    public List<Vertex> getNeighboursAsVertices(Vertex vertex) {
        return getNeighboursAsVertices(vertex.getIndex());
    }

    /**
     * Gibt die Ecken, von denen Kanten zu dieser Ecke hin laufen, als Liste von Ecken zurück.
     *
     * @param index
     *            Index der Ecke.
     * @return Liste mit den Nachbarn.
     */
    public List<Vertex> getNeighboursBackwardsAsVertices(int index) {
        List<Vertex> neighboursBackwards = new ArrayList<>();

        for (int i=0; i<vertices.size(); ++i) {
            if (i == index) {
                continue;
            }
            AdjacenceList list = adjacenceLists.get(i);
            for (AdjacenceEntry entry : list) {
                if (entry.getNeighbourIndex() == index) {
                    Vertex backwardsNeighbour = vertices.get(i);
                    neighboursBackwards.add(backwardsNeighbour);
                }
            }
        }

        return neighboursBackwards;
    }

    /**
     * Gibt die Ecken, von denen Kanten zu dieser Ecke hin laufen, als Liste von Ecken zurück.
     *
     * @param vertex
     *            Ausgangsecke
     * @return Liste mit den Nachbarn.
     */
    public List<Vertex> getNeighboursBackwardsAsVertices(Vertex vertex) {
        return getNeighboursBackwardsAsVertices(vertex.getIndex());
    }

    /**
     * Gibt die Ecken, zu denen Kanten zu dieser Ecke hin und von denen Kanten zu dieser Ecke
     * zurück laufen, als Liste von Ecken zurück.
     *
     * @param vertex
     *            Ausgangsecke.
     * @return Liste mit den Nachbarn.
     */
    public List<Vertex> getForwardsAndBackwardsNeighboursAsVertices(Vertex vertex) {
        return getForwardsAndBackwardsNeighboursAsVertices(vertex.getIndex());
    }

    /**
     * Gibt die Ecken, zu denen Kanten zu dieser Ecke hin und von denen Kanten zu dieser Ecke
     * zurück laufen, als Liste von Ecken zurück.
     *
     * @param index
     *            Index der Ausgangsecke.
     * @return Liste mit den Nachbarn.
     */
    private List<Vertex> getForwardsAndBackwardsNeighboursAsVertices(int index) {
        List<Vertex> forwardsNeighbours = getNeighboursAsVertices(index);
        List<Vertex> backwardsNeighbours = getNeighboursBackwardsAsVertices(index);

        List<Vertex> neighbours = new ArrayList<>();
        for (Vertex vertex : forwardsNeighbours) {
            neighbours.add(vertex);
        }
        for (Vertex vertex : backwardsNeighbours) {
            if (!forwardsNeighbours.contains(vertex)) {
                neighbours.add(vertex);
            }
        }

        return neighbours;
    }

    /** Getter für das Objekt mit den Metadaten. */
    public MetaData getMeta() {
        return meta;
    }

    /** Setter für das Objekt mit den Metadaten. */
    public void setMeta(MetaData meta) {
        this.meta = meta;
    }

    /**
     * Lädt den Graphen aus dem .ged-Format und ersetzt den aktuellen Graphen
     * durch den geladenen.
     *
     * @param text
     *            Eingelesene Datei als ein String.
     */
    public void loadFromGed1Format(String text) {
        Ged1FormatLoader loader = new Ged1FormatLoader(this);
        loader.loadFromSaveString(text);
    }


    /** Gibt den Graphen als String gespeichert im ged-Format zurück. */
    public String saveToGed1Format() {
        Ged1FormatSaver saver = new Ged1FormatSaver(this);
        return saver.createSaveString();
    }

    /**
     * Lädt den Graphen aus dem .ged2-Format und ersetzt den aktuellen Graphen
     * durch den geladenen.
     *
     * @param text
     *            Eingelesene Datei als ein String.
     */
    public void loadFromGed2Format(String text) {
        Ged2FormatLoader loader = new Ged2FormatLoader(this);
        loader.loadFromSaveString(text);
    }


    /** Gibt den Graphen als String gespeichert im ged2-Format zurück. */
    public String saveToGed2Format() {
        Ged2FormatSaver saver = new Ged2FormatSaver(this);
        return saver.createSaveString();
    }

    /**
     * Lädt den Graphen aus dem .ged3-Format und ersetzt den aktuellen Graphen
     * durch den geladenen.
     *
     * @param text
     *            Eingelesene Datei als ein String.
     */
    public void loadFromGed3Format(String text) {
        Ged3FormatLoader loader = new Ged3FormatLoader(this);
        loader.loadFromSaveString(text);
    }


    /** Gibt den Graphen als String gespeichert im ged3-Format zurück. */
    public String saveToGed3Format() {
        Ged3FormatSaver saver = new Ged3FormatSaver(this);
        return saver.createSaveString();
    }

    /**
     * Lädt den Graphen aus dem .ged4-Format und ersetzt den aktuellen Graphen
     * durch den geladenen.
     *
     * @param text
     *            Eingelesene Datei als ein String.
     */
    public void loadFromGed4Format(String text) {
        Ged4FormatLoader loader = new Ged4FormatLoader(this);
        loader.loadFromSaveString(text);
    }

    /** Gibt den Graphen als String gespeichert im ged4-Format zurück. */
    public String saveToGed4Format() {
        Ged4FormatSaver saver = new Ged4FormatSaver(this);
        return saver.createSaveString();
    }

    /** Löscht alle Daten und setzt den Graphen auf den leeren Graphen zurück. */
    public void clearGraph() { // TODO per Interfaces nur den Lademethoden zur Verfügung stellen.
        meta.clear();
        vertices.clear();
        adjacenceLists.clear();
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result
                + ((adjacenceLists == null) ? 0 : adjacenceLists.hashCode());
        result = prime * result + ((meta == null) ? 0 : meta.hashCode());
        result = prime * result
                + ((vertices == null) ? 0 : vertices.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        Graph other = (Graph) obj;
        if (adjacenceLists == null) {
            if (other.adjacenceLists != null)
                return false;
        }
        else if (!adjacenceLists.equals(other.adjacenceLists))
            return false;
        if (meta == null) {
            if (other.meta != null)
                return false;
        }
        else if (!meta.equals(other.meta))
            return false;
        if (vertices == null) {
            if (other.vertices != null)
                return false;
        }
        else if (!vertices.equals(other.vertices))
            return false;
        return true;
    }

    /** Bewegt den Graphen um den Gitterabstand nach links. */
    public void moveLeft() {
        move(-meta.getGridDistance(), 0);
    }

    /** Bewegt den Graphen um den Gitterabstand nach rechts. */
    public void moveRight() {
        move(meta.getGridDistance(), 0);
    }

    /** Bewegt den Graphen um den Gitterabstand nach oben. */
    public void moveUp() {
        move(0, -meta.getGridDistance());
    }

    /** Bewegt den Graphen um den Gitterabstand nach unten. */
    public void moveDown() {
        move(0, meta.getGridDistance());
    }

    /** Bewegt den Graphen um die angegebene Anzahl von Pixeln. */
    private void move(int xDistance, int yDistance) {
        for (Vertex vertex : vertices) {
            PixelPoint coord = vertex.getCoordinates();
            int x = coord.getX() + xDistance;
            int y = coord.getY() + yDistance;
            coord.setX(x);
            coord.setY(y);
        }
    }

    /**
     * Zentriert den Graphen auf den Mittelpunkt der sichtbaren Fläche.
     *
     * @param minX
     *            x-Koordinate der linken oberen Ecke der Fläche.
     * @param minY
     *            y-Koordinate der linken oberen Ecke der Fläche.
     * @param maxX
     *            x-Koordinate der rechten unteren Ecke der Fläche.
     * @param maxY
     *            y-Koordinate der rechten unteren Ecke der Fläche.
     */
    public void centerGraph(int minX, int minY, int maxX, int maxY) {
        if (vertices.isEmpty()) {
            return;
        }

        /* Mittelpunkt der Fläche berechnen: */
        int centerX = (int) ((maxX + minX) / 2.0);
        int centerY = (int) ((maxY + minY) / 2.0);

        /* Extremale Koordinaten des Graphen bestimmen: */
        int vertexMinX = calculateMinimalX();
        int vertexMinY = calculateMinimalY();
        int vertexMaxX = calculateMaximalX();
        int vertexMaxY = calculateMayimalY();

        /* Mittelpunkt des Graphen berechnen: */
        double vertexCenterX = (vertexMaxX + vertexMinX) / 2.0;
        double vertexCenterY = (vertexMaxY + vertexMinY) / 2.0;

        /* Koordinatentransformation: */
        double addX = centerX - vertexCenterX;
        double addY = centerY - vertexCenterY;
        for (Vertex vertex : vertices) {
            PixelPoint p = vertex.getCoordinates();
            int x = p.getX();
            int y = p.getY();
            x += addX;
            y += addY;
            p.setX(x);
            p.setY(y);
        }
    }

    /** Berechnet den kleinsten x-Koordinaten-Wert aller Ecken. */
    private int calculateMinimalX() {
        int minX = Integer.MAX_VALUE;
        for (Vertex vertex : vertices) {
            PixelPoint p = vertex.getCoordinates();
            int x = p.getX();
            if (x < minX) {
                minX = x;
            }
        }
        return minX;
    }

    /** Berechnet den größten x-Koordinaten-Wert aller Ecken. */
    private int calculateMaximalX() {
        int maxX = Integer.MIN_VALUE;
        for (Vertex vertex : vertices) {
            PixelPoint p = vertex.getCoordinates();
            int x = p.getX();
            if (x > maxX) {
                maxX = x;
            }
        }
        return maxX;
    }

    /** Berechnet den kleinsten y-Koordinaten-Wert aller Ecken. */
    private int calculateMinimalY() {
        int minY = Integer.MAX_VALUE;
        for (Vertex vertex : vertices) {
            PixelPoint p = vertex.getCoordinates();
            int y = p.getY();
            if (y < minY) {
                minY = y;
            }
        }
        return minY;
    }

    /** Berechnet den größten y-Koordinaten-Wert aller Ecken. */
    private int calculateMayimalY() {
        int maxY = Integer.MIN_VALUE;
        for (Vertex vertex : vertices) {
            PixelPoint p = vertex.getCoordinates();
            int y = p.getY();
            if (y > maxY) {
                maxY = y;
            }
        }
        return maxY;
    }

    /** Berechnet den größten Radius aller Ecken. */
    private int calculateMinimalRadius() {
        int maxRadius = Integer.MIN_VALUE;
        for (Vertex vertex : vertices) {
            int radius = vertex.getRadius();
            if (maxRadius < radius) {
                maxRadius = radius;
            }
        }
        return maxRadius;
    }

    /**
     * Zentriert den Graphen auf den Mittelpunkt der sichtbaren Fläche und vergrößert ihn so weit
     * er dann noch auf den Schirm passt.
     *
     * @param minX
     *            x-Koordinate der linken oberen Ecke der Fläche.
     * @param minY
     *            y-Koordinate der linken oberen Ecke der Fläche.
     * @param maxX
     *            x-Koordinate der rechten unteren Ecke der Fläche.
     * @param maxY
     *            y-Koordinate der rechten unteren Ecke der Fläche.
     */
    public void maximizeGraph(int minX, int minY, int maxX, int maxY) {
        boolean respectingAspectRatio = false;
        boolean centerAroundVertex = false;
        Vertex unusedCenter = new Vertex();
        maximizeGraph(minX, minY, maxX, maxY, respectingAspectRatio, centerAroundVertex,
                unusedCenter);
    }

    /**
     * Zentriert den Graphen auf den Mittelpunkt der sichtbaren Fläche und vergrößert ihn so weit
     * er dann noch auf den Schirm passt.
     *
     * @param minX
     *            x-Koordinate der linken oberen Ecke der Fläche.
     * @param minY
     *            y-Koordinate der linken oberen Ecke der Fläche.
     * @param maxX
     *            x-Koordinate der rechten unteren Ecke der Fläche.
     * @param maxY
     *            y-Koordinate der rechten unteren Ecke der Fläche.
     */
    public void maximizeGraphRespectingAspectRatio(int minX, int minY, int maxX, int maxY) {
        boolean respectingAspectRatio = true;
        boolean centerAroundVertex = false;
        Vertex unusedCenter = new Vertex();
        maximizeGraph(minX, minY, maxX, maxY, respectingAspectRatio, centerAroundVertex,
                unusedCenter);
    }

    /**
     * Zentriert den Graphen auf den Mittelpunkt der sichtbaren Fläche und vergrößert ihn so weit
     * er dann noch auf den Schirm passt.
     *
     * @param minX
     *            x-Koordinate der linken oberen Ecke der Fläche.
     * @param minY
     *            y-Koordinate der linken oberen Ecke der Fläche.
     * @param maxX
     *            x-Koordinate der rechten unteren Ecke der Fläche.
     * @param maxY
     *            y-Koordinate der rechten unteren Ecke der Fläche.
     * @param center
     *            Ecke die in den Mittelpunkt gerückt wird.
     */
    public void maximizeGraphRespectingAspectRatio(int minX, int minY, int maxX, int maxY,
            Vertex center) {
        boolean respectingAspectRatio = true;
        boolean centerAroundVertex = true;
        maximizeGraph(minX, minY, maxX, maxY, respectingAspectRatio, centerAroundVertex, center);
    }

    private void maximizeGraph(int minX, int minY, int maxX, int maxY,
            boolean respectingAspectRatio, boolean centerAroundVertex, Vertex center) {
        if (vertices.isEmpty()) {
            return;
        }

        int maxRadius = calculateMinimalRadius();

        /* Extreme erlaubte Werte bestimmen: */
        int delta = 20;
        if (maxRadius >= delta - 2) {
            delta = maxRadius + 10;
            // TODO Beschriftungen und so weiter können trotzdem außerhalb liegen.
        }
        int screenMinX = minX + delta;
        int screenMinY = minY + delta;
        int screenMaxX = maxX - delta;
        int screenMaxY = maxY - delta;

        /* Extremale Koordinaten des Graphen bestimmen: */
        int vertexMinX = calculateMinimalX();
        int vertexMinY = calculateMinimalY();
        int vertexMaxX = calculateMaximalX();
        int vertexMaxY = calculateMayimalY();

        if (centerAroundVertex) {
            int middleX = (vertexMaxX - vertexMinX) / 2;
            int middleY = (vertexMaxY - vertexMinY) / 2;

            int wantedCenterX = center.getX();
            int wantedCenterY = center.getY();

            int moveX = wantedCenterX - middleX;
            int moveY = wantedCenterY - middleY;

            if (moveX > 0) { // TODO eventuell * 2 überall ...
                vertexMaxX += 2 * moveX;
            }
            else {
                vertexMinX += 2 * moveX; // moveX ist dann ja negativ.
            }
            if (moveY > 0) {
                vertexMaxY += 2 * moveY;
            }
            else {
                vertexMinY += 2 * moveY; // moveY ist dann ja negativ.
            }
        }

        /* Berechnung der Faktoren für die Koordinatentransformation: */
        double xFactor;
        if (vertexMaxX == vertexMinX) {
            xFactor = 1.0;
        }
        else {
            xFactor = (screenMaxX - screenMinX) / (1.0 * vertexMaxX - vertexMinX);
        }
        double yFactor;
        if (vertexMaxY == vertexMinY) {
            yFactor = 1.0;
        }
        else {
            yFactor = (screenMaxY - screenMinY) / (1.0 * vertexMaxY - vertexMinY);
        }

        if (respectingAspectRatio) {
            double minFactor = Math.min(xFactor, yFactor);
            xFactor = minFactor;
            yFactor = minFactor;
        }

        /* Koordinatentransformation: */
        for (Vertex vertex : vertices) {
            PixelPoint p = vertex.getCoordinates();
            double x = p.getX();
            double y = p.getY();
            if (vertexMaxX == vertexMinX) {
                x = (screenMinX + screenMaxX) / 2.0;
            }
            else {
                x = screenMinX + (x - vertexMinX) * xFactor;
            }
            if (vertexMaxY == vertexMinY) {
                y = (screenMinY + screenMaxY) / 2.0;
            }
            else {
                y = screenMinY + (y - vertexMinY) * yFactor;
            }
            p.setX(x);
            p.setY(y);
        }

        /* Gitterweite mit verändern: */
        double distance = meta.getGridDistance();
        if (xFactor > yFactor) {
            distance *= xFactor;
        }
        else {
            distance *= yFactor;
        }
        meta.setGridDistance((int) distance);

        /*
         * Auf automatisches Einrasten an dieser Stelle wird verzichtet,
         * da der Benutzer dies selbst machen kann, wenn gewünscht.
         */
    }

    /**
     * Verkleinert den Graphen.
     *
     * @param minX
     *            x-Koordinate der linken oberen Ecke der Fläche.
     * @param minY
     *            y-Koordinate der linken oberen Ecke der Fläche.
     * @param maxX
     *            x-Koordinate der rechten unteren Ecke der Fläche.
     * @param maxY
     *            y-Koordinate der rechten unteren Ecke der Fläche.
     */
    public void reduceGraph(int minX, int minY, int maxX, int maxY) {
        if (vertices.isEmpty()) {
            return;
        }

        /* Extremale Koordinaten des Graphen bestimmen: */
        int vertexMinX = calculateMinimalX();
        int vertexMinY = calculateMinimalY();
        int vertexMaxX = calculateMaximalX();
        int vertexMaxY = calculateMayimalY();

        /* Mittelpunkt des Graphen berechnen: */
        double vertexCenterX = (vertexMaxX + vertexMinX) / 2.0;
        double vertexCenterY = (vertexMaxY + vertexMinY) / 2.0;

        /* Koordinatentransformation: */
        double factor = 0.8; // 0.66666
        for (Vertex vertex : vertices) {
            PixelPoint p = vertex.getCoordinates();
            double x = p.getX();
            double y = p.getY();
            x = vertexCenterX + (x - vertexCenterX) * factor;
            y = vertexCenterY + (y - vertexCenterY) * factor;
            p.setX(x);
            p.setY(y);
        }

        /* Gitterweite mit verändern: */
        double distance = meta.getGridDistance();
        distance *= factor;
        meta.setGridDistance((int) distance);
    }

    /** Vergrößert den Graphen.
     *
     * @param minX
     *            x-Koordinate der linken oberen Ecke der Fläche.
     * @param minY
     *            y-Koordinate der linken oberen Ecke der Fläche.
     * @param maxX
     *            x-Koordinate der rechten unteren Ecke der Fläche.
     * @param maxY
     *            y-Koordinate der rechten unteren Ecke der Fläche.
     */
    public void enlargeGraph(int minX, int minY, int maxX, int maxY) {
        if (vertices.isEmpty()) {
            return;
        }
        /* Extremale Koordinaten des Graphen bestimmen: */
        int vertexMinX = calculateMinimalX();
        int vertexMinY = calculateMinimalY();
        int vertexMaxX = calculateMaximalX();
        int vertexMaxY = calculateMayimalY();

        /* Mittelpunkt des Graphen berechnen: */
        double vertexCenterX = (vertexMaxX + vertexMinX) / 2.0;
        double vertexCenterY = (vertexMaxY + vertexMinY) / 2.0;

        /* Koordinatentransformation: */
        double factor = 1.25; // 1.5
        for (Vertex vertex : vertices) {
            PixelPoint p = vertex.getCoordinates();
            double x = p.getX();
            double y = p.getY();
            x = vertexCenterX + (x - vertexCenterX) * factor;
            y = vertexCenterY + (y - vertexCenterY) * factor;
            p.setX(x);
            p.setY(y);
        }

        /* Gitterweite mit verändern: */
        double distance = meta.getGridDistance();
        distance *= factor;
        meta.setGridDistance((int) distance);
    }

    /** Lässt alle Ecken des Graphen im Gitter einrasten. */
    public void snapAllVerticesToGrid() {
        for (Vertex vertex : vertices) {
            snapVertexToGrid(vertex);
        }
    }

    /**
     * Testet, ob diese Ecke sich in ihrer Darstellung mit einer anderen Ecke überschneidet.
     *
     * @param vertex
     *            Zu prüfende Ecke.
     * @return Wahrheitswert true bedeutet, sie überschneidet sich mit mindestens einer anderen
     *         Ecke.
     */
    public boolean vertexIntersctsWithOtherVertex(Vertex vertex) {
        for (Vertex otherVertex : vertices) {
            /* Kein Vergleich mit sich selbst! */
            if (vertex != otherVertex) {
                /*
                 * Wir prüfen, ob die Summe der Radien kürzer oder so lang ist wie
                 * Verbindungslinien zwischen den zwei Ecken:
                 */
                int ourRadius = vertex.getRadius();
                int otherRadius = otherVertex.getRadius();
                PixelPoint ourCenter = vertex.getCoordinates();
                PixelPoint otherCenter = otherVertex.getCoordinates();
                Line line = new Line(ourCenter.toMathPoint(), otherCenter.toMathPoint());
                if (line.getDistance() <= ourRadius + otherRadius) {
                    return true;
                }
            }
        }
        return false;
    }

    /** Setzt die Farben aller Ecken auf Default. */
    public void setAllVertexColorsToDefault() {
        for (Vertex vertex : vertices) {
            vertex.setColor(GedColor.DEFAULT);
        }
    }

    /** Setzt die Radien aller Ecken auf Default. */
    public void setAllVertexRadiusToDefault() {
        for (Vertex vertex : vertices) {
            vertex.setRadiusToDefault();
        }
    }

    /** Setzt die Farben aller Kanten auf Default. */
    public void setAllEdgeColorsToDefault() {
        for (int startIndex = 0; startIndex < getNumberOfVertices(); ++startIndex) {
            AdjacenceList list = getNeighbours(startIndex);
            for (AdjacenceEntry entry : list) {
                entry.setColor(GedColor.DEFAULT);
            }
        }
    }

    /** Setzt die Dicken aller Kanten auf Default. */
    public void setAllEdgeWidthsToDefault() {
        for (int startIndex = 0; startIndex < getNumberOfVertices(); ++startIndex) {
            AdjacenceList list = getNeighbours(startIndex);
            for (AdjacenceEntry entry : list) {
                entry.setWidthToDefault();
            }
        }
    }

    /** Getter für die Kantenmenge des Graphen in Form von Adjazenzlisten. */
    public AdjacenceLists getAdjacenceLists() {
        return adjacenceLists;
    }

    /** Getter für die Kantenmenge des Graphen zu einer Ecke in Form einer Adjazenzliste. */
    public AdjacenceList getAdjacenceList(Vertex vertex) {
        return adjacenceLists.get(vertex.getIndex());
    }

    public Graph createEdgeSubGraph(int startIndex, AdjacenceEntry entry, int subGraphSize) {
        return createEdgeSubGraph(startIndex, entry, subGraphSize, subGraphSize);
    }

    public Graph createEdgeSubGraph(int startIndex, AdjacenceEntry entry, int subGraphSizeX,
            int subGraphSizeY) {
        EdgeSubGraphCreator creator = new EdgeSubGraphCreator(this, subGraphSizeX, subGraphSizeY);
        return creator.createOneEdgeSubGraph(startIndex, entry);
    }

    public Graph createVertexWithNeighboursSubGraph(Vertex vertex, int subGraphSize) {
        return createVertexWithNeighboursSubGraph(vertex.getIndex(), subGraphSize);
    }

    public Graph createVertexWithNeighboursSubGraph(int startIndex, int subGraphSize) {
        return createVertexWithNeighboursSubGraph(startIndex, subGraphSize, subGraphSize);
    }

    public Graph createVertexWithNeighboursSubGraph(int startIndex, int subGraphSizeX,
            int subGraphSizeY) {
        VertexWithItsNeigboursSubGraphCreator creator = new VertexWithItsNeigboursSubGraphCreator(
                this, subGraphSizeX, subGraphSizeY);
        return creator.createVertexWithItsNeigboursSubGraph(startIndex);
    }

}
