package de.duehl.math.graph.ged.ui.creation.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JToggleButton;

import de.duehl.math.graph.ged.logic.Logic;
import de.duehl.math.graph.ged.resources.IconDefinitions;
import de.duehl.math.graph.ged.ui.GedWindowListener;
import de.duehl.math.graph.ged.ui.GuiElements;
import de.duehl.math.graph.ged.ui.elements.ModusButtonMouseListener;
import de.duehl.math.graph.ged.ui.graphpanel.EditModus;
import de.duehl.math.graph.ged.ui.graphpanel.GraphPanel;
import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.layout.VerticalLayout;
import de.duehl.swing.ui.resources.IconLoader;

/**
 * Diese Klasse baut die Elemente der grafischen Oberfläche auf.
 *
 * @version 1.01     2019-05-22
 * @author Christian Dühl
 */

public class CreateGui {

    /** Enthält die grafischen Elemente. */
    private GuiElements elements;

    /** Zugriff auf die Programmlogik */
    private Logic logic;

    /**
     * Konstruktor
     *
     * @param width
     *            Soll-Breite des Graphenpanels.
     * @param height
     *            Soll-Höhe des Graphenpanels.
     * @param elements
     *            Enthält die grafischen Elemente, die man für späteren Zugriff
     *            benötigt.
     * @param logic
     *            Zugriff auf die Programmlogik
     */
    public CreateGui(int width, int height, GuiElements elements, Logic logic) {
        this.logic       = logic;
        this.elements    = elements;
        createGui(width, height);
    }

    private Icon loadIcon(String imageIdentifier) {
        IconDefinitions iconDefinitions = new IconDefinitions();
        IconLoader iconLoader = iconDefinitions.createIconLoader();
        return iconLoader.loadIcon(imageIdentifier);
    }

    /** Erzeugt die grafische Oberfläche. */
    private void createGui(int width, int height) {

        JFrame frame = elements.getFrame();
        frame.setLocation(100, 100);
        //frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        frame.addWindowListener(new GedWindowListener(logic));
        frame.setLayout(new BorderLayout());

        frame.add(createGraphPanel(width, height), BorderLayout.CENTER);
        frame.add(createUpperPanel(), BorderLayout.NORTH);
        frame.add(createRightPanel(), BorderLayout.EAST);
        frame.add(createLowerPanel(), BorderLayout.SOUTH);
    }

    private JComponent createGraphPanel(int width, int height) {
        GraphPanel graphPanel = new GraphPanel(width, height, logic, elements);
        GuiTools.createTitle(graphPanel);

        Dimension dimension = new Dimension(width, height);
        graphPanel.setPreferredSize(dimension);
        graphPanel.setMinimumSize(dimension);

        graphPanel.setBackground(Color.WHITE);

        graphPanel.addMouseListener(graphPanel);
        graphPanel.doAddMouseMotionListener();
        elements.setGraphPanel(graphPanel);

        return graphPanel;
    }

    /** Erzeugt den oberen Bereich. */
    private JComponent createUpperPanel() {
        JPanel upperPanel = new JPanel();
        upperPanel.setLayout(new GridBagLayout());

        /*
         * Vor Aufbau des Modus-Panels den Label modusDescription erzeugen, da dieser in
         * createModusButtonPanel() benötigt wird:
         */
        createModusDecriptionLabel();

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets     = new Insets(0, 0, 0, 0);
        gbc.fill       = GridBagConstraints.BOTH;
        gbc.anchor     = GridBagConstraints.WEST;
        gbc.weightx    = 0.0;
        gbc.weighty    = 0.0;
        gbc.gridy      = 0;

        /* Dateioperationen: */
        gbc.gridx      = 0;
        upperPanel.add(createFileOperationsPanel(), gbc);

        /* Historie: */
        gbc.insets     = new Insets(0, 10, 0, 0);
        gbc.gridx      = 1;
        upperPanel.add(createHistoryPanel(), gbc);

        /* Modus-Panel einfügen: */
        gbc.insets     = new Insets(0, 10, 0, 0);
        gbc.gridx      = 2;
        upperPanel.add(createModusButtonPanel(), gbc);

        /* Modusbeschreibungs-Panel einfügen: */
        gbc.gridx      = 3;
        gbc.weightx    = 1.0;
        gbc.insets     = new Insets(0, 5, 0, 0);
        upperPanel.add(createModusButtonShowDescriptionPanel(), gbc);

        return upperPanel;
    }

    private void createModusDecriptionLabel() {
        JLabel modusDescriptionLabel = new JLabel();
        modusDescriptionLabel.setForeground(Color.BLUE);
        elements.setModusDescriptionLabel(modusDescriptionLabel);
    }

    /** Erzeugt das Panel mit den Dateioperationen. */
    private JComponent createFileOperationsPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(createGridLayoutForGraphButtons());
        GuiTools.createTitle(panel);

        panel.add(createNewGraphButton());
        panel.add(createLoadGed4Button());
        panel.add(createSaveGed4Button());
        panel.add(createSaveAsGed4Button());

        return panel;
    }

    private JComponent createNewGraphButton() {
        return createPicturedButton(IconDefinitions.FILE_NEW, e -> logic.createEmptyFile());
    }

    private JComponent createLoadGed4Button() {
        return createPicturedButton(IconDefinitions.FILE_LOAD, e -> logic.loadGed4File());
    }

    private JComponent createSaveGed4Button() {
        return createPicturedButton(IconDefinitions.FILE_SAVE, e -> logic.saveGed4File());
    }

    private JComponent createSaveAsGed4Button() {
        return createPicturedButton(IconDefinitions.FILE_SAVE_AS, e -> logic.saveGed4FileAs());
    }

    /** Erzeugt das Panel mit den Historie-Operationen. */
    private JComponent createHistoryPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(createGridLayoutForGraphButtons());
        GuiTools.createTitle(panel);

        panel.add(createPreviousHistoryGraphButton());
        panel.add(createNextHistoryGraphButton());

        return panel;
    }

    private JComponent createPreviousHistoryGraphButton() {
        JButton previousButton = createDisabledPicturedButton(IconDefinitions.HISTORY_PREVIOUS,
                e -> logic.undo());
        elements.setPreviousButton(previousButton);
        return previousButton;
    }

    private JComponent createNextHistoryGraphButton() {
        JButton button = createDisabledPicturedButton(IconDefinitions.HISTORY_NEXT,
                e -> logic.redo());
        elements.setNextButton(button);
        return button;
    }

    /**
     * Erzeugt das Modus-Button-Panel.
     *
     * Da Swing anders als Perl-Tk keine "vernünftigen" Buttons mit Bildern ohne großartigen Rahmen
     * anbietet, wird der gedrückt / nicht gedrückt Status der Button-Gruppe selbst mit
     * MouseListenern und BevelBordern, die wechselseitig "raised" bzw. "lowered" sind,
     * dargestellt. Der ausgewählte Status wird außerdem in modus im Graphpanel festgehalten.
     */
    private JPanel createModusButtonPanel() {
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridBagLayout());
        GuiTools.createTitle(buttonPanel);

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets     = new Insets(0, 0, 0, 0);
        gbc.fill       = GridBagConstraints.BOTH;
        gbc.anchor     = GridBagConstraints.NORTHWEST;
        gbc.weightx    = 0.0;
        gbc.weighty    = 0.0;
        gbc.gridx      = 0;
        gbc.gridy      = 0;

        ButtonGroup group = new ButtonGroup();

        for (EditModus modus : EditModus.values()) {
            if (gbc.gridx == 3 || gbc.gridx == 4 || gbc.gridx == 5) {
                gbc.insets = new Insets(0, 7, 0, 0);
            }
            else {
                gbc.insets = new Insets(0, 2, 0, 0);
            }

            JToggleButton button = modus.getToggleButton();
            button.addMouseListener(new ModusButtonMouseListener(modus, elements));

            group.add(button);
            buttonPanel.add(button, gbc);
            ++gbc.gridx;
        }

        activateModusButton(EditModus.ADD_VERTEX);

        return buttonPanel;
    }

    private void activateModusButton(EditModus modus) {
        JToggleButton button = modus.getToggleButton();
        button.setBorder(ModusButtonMouseListener.LOWERED_BORDER);
        button.setSelected(true);
        GraphPanel graphPanel = elements.getGraphPanel();
        graphPanel.setModus(modus);

        JLabel modusDescriptionLabel = elements.getModusDescriptionLabel();
        modusDescriptionLabel.setText(modus.getDescription());
    }

    private JComponent createModusButtonShowDescriptionPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        GuiTools.createTitle(panel);

        panel.add(elements.getModusDescriptionLabel(), BorderLayout.CENTER);

        return panel;
    }

    private JPanel createRightPanel() {
        JPanel rightPanel = new JPanel();
        rightPanel.setLayout(new BorderLayout());
        GuiTools.createTitle(rightPanel);

        rightPanel.add(createRightUpperPart(), BorderLayout.NORTH);
        rightPanel.add(createRightLowerPart(), BorderLayout.SOUTH);

        return rightPanel;
    }

    private JComponent createRightUpperPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new VerticalLayout(5, VerticalLayout.BOTH));

        panel.add(createDeleteGraphButton());
        panel.add(createGraphLeftUpDownRightPart());
        panel.add(createGraphCenterMaxSmallerBiggerPart());

        return panel;
    }

    private JComponent createDeleteGraphButton() {
        JButton deleteAll = new JButton("Löschen");
        deleteAll.addActionListener(e -> logic.createEmptyFile());
        return deleteAll;
    }

    /* zweite Reihe: links oben unten rechts */
    private JComponent createGraphLeftUpDownRightPart() {
        JPanel panel = new JPanel();
        panel.setLayout(createGridLayoutForGraphButtons());

        panel.add(createMoveGraphLeftButton());
        panel.add(createMoveGraphUpButton());
        panel.add(createMoveGraphDownButton());
        panel.add(createMoveGraphRightButton());

        return panel;
    }

    private GridLayout createGridLayoutForGraphButtons() {
        return new GridLayout(1, 0, 2, 2);
    }

    private JComponent createMoveGraphLeftButton() {
        return createPicturedButton(IconDefinitions.MOVE_GRAPH_LEFT, e -> logic.moveLeft());
    }

    private JComponent createMoveGraphUpButton() {
        return createPicturedButton(IconDefinitions.MOVE_GRAPH_UP, e -> logic.moveUp());
    }

    private JComponent createMoveGraphDownButton() {
        return createPicturedButton(IconDefinitions.MOVE_GRAPH_DOWN, e -> logic.moveDown());
    }

    private JComponent createMoveGraphRightButton() {
        return createPicturedButton(IconDefinitions.MOVE_GRAPH_RIGHT, e -> logic.moveRight());
    }

    /* dritte Reihe: +  # Lupe- Lupe+ */
    private JComponent createGraphCenterMaxSmallerBiggerPart() {
        JPanel panel = new JPanel();
        panel.setLayout(createGridLayoutForGraphButtons());

        panel.add(createCenterGraphButton());
        panel.add(createMaximizeGraphButton());
        panel.add(createReduceGraphButton());
        panel.add(createEnlargeGraphButton());

        return panel;
    }

    private JComponent createCenterGraphButton() {
        return createPicturedButton(IconDefinitions.CENTER_GRAPH, e -> centerGraph());
    }

    private JComponent createMaximizeGraphButton() {
        return createPicturedButton(IconDefinitions.MAXIMIZE_GRAPH, e -> maximizeGraph());
    }

    private JComponent createReduceGraphButton() {
        return createPicturedButton(IconDefinitions.SMALLER_GRAPH, e -> reduceGraph());
    }

    private JComponent createEnlargeGraphButton() {
        return createPicturedButton(IconDefinitions.BIGGER_GRAPH, e -> enlargeGraph());
    }

    private JButton createDisabledPicturedButton(String pictureIdentifier, ActionListener listener) {
        JButton button = createPicturedButton(pictureIdentifier, listener);
        button.setEnabled(false);
        return button;
    }

    private JButton createPicturedButton(String pictureIdentifier, ActionListener listener) {
        JButton button = new JButton(loadIcon(pictureIdentifier));
        button.setBorder(BorderFactory.createRaisedBevelBorder());
        button.addActionListener(listener);
        return button;
    }

    private void centerGraph() {
        int width = elements.getGraphPanel().getWidth();
        int height = elements.getGraphPanel().getHeight();
        logic.centerGraph(0, 0, width, height);
    }

    private void maximizeGraph() {
        int width = elements.getGraphPanel().getWidth();
        int height = elements.getGraphPanel().getHeight();
        logic.maximizeGraph(0, 0, width, height);
    }

    private void reduceGraph() {
        int width = elements.getGraphPanel().getWidth();
        int height = elements.getGraphPanel().getHeight();
        logic.reduceGraph(0, 0, width, height);
    }

    private void enlargeGraph() {
        int width = elements.getGraphPanel().getWidth();
        int height = elements.getGraphPanel().getHeight();
        logic.enlargeGraph(0, 0, width, height);
    }

    private JComponent createRightLowerPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new GridBagLayout());
        //panel.setLayout(createGrigLayoutForRightGraphButtons());


        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets     = new Insets(0, 0, 0, 0);
        gbc.fill       = GridBagConstraints.NONE;
        gbc.anchor     = GridBagConstraints.WEST;
        gbc.weightx    = 0.0;
        gbc.weighty    = 0.0;
        gbc.gridx = 0;

        /* erste Reihe: neu Zeichnen */
        {
            gbc.gridy = 0;
            // TODO
        }

        /* zweite Reihe: Beenden */
        {
            gbc.gridy = 1;
            // TODO
        }

        return panel;
    }

    /** Erzeugt den unteren Panel. */
    private JPanel createLowerPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        //GuiTools.createTitle(panel);

        panel.add(createStausPanel(), BorderLayout.CENTER);
        panel.add(createSizeAndChangePart(), BorderLayout.EAST);

        return panel;
    }

    private JComponent createStausPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        GuiTools.createTitle(panel);

        panel.add(createStatusLabel(), BorderLayout.CENTER);

        return panel;
    }

    private JComponent createStatusLabel() {
        JLabel statusLabel = new JLabel(" ");
        elements.setStatusLabel(statusLabel);
        return statusLabel;
    }

    private JComponent createSizeAndChangePart() {
        JPanel lowerRightPanel = new JPanel();
        lowerRightPanel.setLayout(new BorderLayout());

        lowerRightPanel.add(createSizePanel(), BorderLayout.WEST);
        lowerRightPanel.add(createChangePanel(), BorderLayout.EAST);//TODO
        return lowerRightPanel;
    }

    /** Erzeugt das Canvasgröße-Anzeige-Panel. */
    private JComponent createSizePanel() {
        JPanel sizePanel = new JPanel();
        sizePanel.setLayout(new BorderLayout());
        GuiTools.createTitle(sizePanel);

        sizePanel.add(createSizeLabel(), BorderLayout.CENTER);

        return sizePanel;
    }

    private JComponent createSizeLabel() {
        JLabel sizeLabel = new JLabel("                ");
        elements.setSizeLabel(sizeLabel);
        return sizeLabel;
    }

    private JComponent createChangePanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        GuiTools.createTitle(panel);

        panel.add(createChangeLabel(), BorderLayout.CENTER);

        return panel;
    }

    private JComponent createChangeLabel() {
        JLabel changeLabel = new JLabel("     ");
        elements.setChangeLabel(changeLabel);
        return changeLabel;
    }

}
