package de.duehl.math.graph.ged.ui.creation.menu;

import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JMenu;
import javax.swing.JMenuItem;

import de.duehl.basics.collections.CollectionsHelper;
import de.duehl.math.graph.ged.logic.Logic;
import de.duehl.math.graph.ged.ui.GuiElements;
import de.duehl.swing.ui.menu.MyCheckBoxMenuItem;
import de.duehl.swing.ui.menu.MyMenuItem;
import de.duehl.swing.ui.menu.SingleMenuCreation;

/**
 * Diese Klasse stellt die Methoden zur Verfügung, um das Bearbeiten-Menü für die grafische
 * Oberfläche zu erstellen.
 *
 * @version 1.01     2019-05-22
 * @author Christian Dühl
 */

public class EditMenuCreation implements SingleMenuCreation {

    /** Verbindung zur Programmlogik. */
    private final Logic logic;

    /** Enthält die grafischen Elemente. */
    private final GuiElements elements;

    /** Tastaturmaske für die Control-Taste. */
    private final int ctrlMask;

    /**
     * Konstruktor.
     *
     * @param logic
     *            Verbindung zur Programmlogik.
     * @param ctrlMask
     *            Tastaturmaske für die Control-Taste.
     */
    public EditMenuCreation(Logic logic, GuiElements elements, int ctrlMask) {
        this.logic = logic;
        this.elements = elements;
        this.ctrlMask = ctrlMask;
    }

    /** Erzeugt das Bearbeitenmenü. */
    @Override
    public JMenu createTheMenu() {
        JMenu menu = new JMenu("Bearbeiten");
        menu.setMnemonic('b');

        menu.add(createUndoMenuItem());
        menu.add(createRepeatMenuItem());

        menu.addSeparator();

        menu.add(createCancelEditMenuItem());

        menu.addSeparator();

        menu.add(createTheMetaSubMenu());

        menu.addSeparator();

        menu.add(createSnappAllVerticesInGridMenuItem());

        menu.addSeparator();

        menu.add(createTheInsertCirclesSubMenu());
        menu.add(createTheInsertCompleteGraphsSubMenu());
        menu.add(createInsertCenterMenuItem());

        menu.addSeparator();

        menu.add(createMoveGraphLeftMenuItem());
        menu.add(createMoveGraphRightMenuItem());
        menu.add(createMoveGraphUpMenuItem());
        menu.add(createMoveGraphDownMenuItem());

        menu.addSeparator();

        menu.add(createSetAllVertexColorsToDefaultMenuItem());
        menu.add(createSetAllVertexRadiusToDefaultMenuItem());
        menu.add(createSetAllEdgeColorsToDefaultMenuItem());
        menu.add(createSetAllEdgeWidthsToDefaultMenuItem());

        return menu;
    }

    private JMenuItem createUndoMenuItem() {
        MyMenuItem menuItem = new MyMenuItem("Rückgängig")
            .mnemonic('ü')
            .accelerator(KeyEvent.VK_Z, ctrlMask)
            .actionListener(e -> logic.undo());

        menuItem.setEnabled(false);
        elements.setPreviousHistoryMenuItem(menuItem); // Für setEnabled()

        return menuItem;
    }

    private JMenuItem createRepeatMenuItem() {
        MyMenuItem menuItem = new MyMenuItem("Wiederholen")
            .mnemonic('w')
            .accelerator(KeyEvent.VK_Y, ctrlMask)
            .actionListener(e -> logic.redo());

        menuItem.setEnabled(false);
        elements.setNextHistoryMenuItem(menuItem); // Für setEnabled()

        return menuItem;
    }

    private JMenuItem createCancelEditMenuItem() {
        return new MyMenuItem("Bearbeitung abbrechen")
            .mnemonic('a')
            //.accelerator(KeyEvent.VK_R, ctrlMask)
            .actionListener(e -> logic.interruptEditing());
    }

    /** Erzeugt das Untermenü für die Metadaten. */
    private JMenu createTheMetaSubMenu() {
        JMenu menu = new JMenu("Metadaten des Graphen ändern");

        menu.setMnemonic('m');
        if (FileMenuCreation.OPAQUE_SUBMENUES) {
            menu.setOpaque(true);
        }

        menu.add(createSelectDefaultVertexColorMenuItem());
        menu.add(createSelectDefaultVertexRadiusMenuItem());

        menu.addSeparator();

        menu.add(createSelectDefaultEdgeColorMenuItem());
        menu.add(createSelectDefaultEdgeWidthMenuItem());

        menu.addSeparator();

        menu.add(createShowGridCheckBoxMenuItem());
        menu.add(createSelectGridDistanceMenuItem());
        menu.add(createSelectGridColorMenuItem());
        menu.add(createSnapInGridCheckBoxMenuItem());

        menu.addSeparator();

        menu.add(createSelectBackgroundColorMenuItem());
        menu.add(createSelectBackgroundPictureMenuItem());
        menu.add(createRemoveBackgroundPictureMenuItem());

        return menu;
    }

    private JMenuItem createSelectDefaultVertexColorMenuItem() {
        return new MyMenuItem("Standardfarbe der Ecken wählen ...")
            .mnemonic('f')
            //.accelerator(KeyEvent.VK_R, ctrlMask)
            .actionListener(e -> logic.chooseDefaultVertexColor());
    }

    private JMenuItem createSelectDefaultVertexRadiusMenuItem() {
        return new MyMenuItem("Standardradius der Ecken wählen ...")
            .mnemonic('r')
            //.accelerator(KeyEvent.VK_R, ctrlMask)
            .actionListener(e -> logic.chooseDefaultVertexRadius());
    }

    private JMenuItem createSelectDefaultEdgeColorMenuItem() {
        return new MyMenuItem("Standardfarbe der Kanten wählen ...")
            .mnemonic('k')
            //.accelerator(KeyEvent.VK_R, ctrlMask)
            .actionListener(e -> logic.chooseDefaultEdgeColor());
    }

    private JMenuItem createSelectDefaultEdgeWidthMenuItem() {
        return new MyMenuItem("Standarddicke der Kanten wählen ...")
            .mnemonic('d')
            //.accelerator(KeyEvent.VK_R, ctrlMask)
            .actionListener(e -> logic.chooseDefaultEdgeWidth());
    }

    private JMenuItem createShowGridCheckBoxMenuItem() {
        MyCheckBoxMenuItem checkBoxMenuItem = new MyCheckBoxMenuItem("Gitternetz anzeigen")
                //.mnemonic('h')
                //.accelerator(KeyEvent.VK_R, ctrlMask)
                .actionListener(e -> logic.toggleMetaShowGrid());
        elements.setShowGridCheckBox(checkBoxMenuItem);
        return checkBoxMenuItem;
    }

    private JMenuItem createSelectGridDistanceMenuItem() {
        return new MyMenuItem("Gitterweite wählen ...")
            .mnemonic('w')
            //.accelerator(KeyEvent.VK_R, ctrlMask)
            .actionListener(e -> logic.chooseGridDistance());
    }

    private JMenuItem createSelectGridColorMenuItem() {
        return new MyMenuItem("Farbe des Gitters wählen ...")
            .mnemonic('g')
            //.accelerator(KeyEvent.VK_R, ctrlMask)
            .actionListener(e -> logic.chooseGridColor());
    }

    private JMenuItem createSnapInGridCheckBoxMenuItem() {
        MyCheckBoxMenuItem checkBoxMenuItem = new MyCheckBoxMenuItem("Am Gitter einrasten")
                //.mnemonic('h')
                //.accelerator(KeyEvent.VK_R, ctrlMask)
                .actionListener(e -> logic.toggleMetaSnapInGrid());
        elements.setSnapInGridCheckBox(checkBoxMenuItem);
        return checkBoxMenuItem;
    }

    private JMenuItem createSelectBackgroundColorMenuItem() {
        return new MyMenuItem("Hintergrundfarbe ...")
            .mnemonic('h')
            //.accelerator(KeyEvent.VK_R, ctrlMask)
            .actionListener(e -> logic.chooseBackgroundColor());
    }

    private JMenuItem createSelectBackgroundPictureMenuItem() {
        return new MyMenuItem("Hinterbild auswählen ...")
            .mnemonic('b')
            //.accelerator(KeyEvent.VK_R, ctrlMask)
            .actionListener(e -> logic.chooseBackgroundPicture());
    }

    private JMenuItem createRemoveBackgroundPictureMenuItem() {
        return new MyMenuItem("Hinterbild entfernen")
            .mnemonic('e')
            //.accelerator(KeyEvent.VK_R, ctrlMask)
            .actionListener(e -> logic.removeBackgroundPicture());
    }

    private JMenuItem createSnappAllVerticesInGridMenuItem() {
        return new MyMenuItem("Alle Ecken am Gitter einrasten")
            .mnemonic('r')
            .accelerator(KeyEvent.VK_R, ctrlMask)
            .actionListener(e -> logic.snapAllVerticesToGrid());
    }

    /** Erzeugt das Untermenü für die Kreise. */
    private JMenu createTheInsertCirclesSubMenu() {
        JMenu menu = new JMenu("Kreise hinzufügen");

        menu.setMnemonic('k');
        if (FileMenuCreation.OPAQUE_SUBMENUES) {
            menu.setOpaque(true);
        }

        List<String> circles = new ArrayList<>();
        List<Integer> numbers = new ArrayList<>();
        circles.add("Dreieck");           numbers.add(3);
        circles.add("Viereck");           numbers.add(4);
        circles.add("Fünfeck");           numbers.add(5);
        circles.add("Sechseck");          numbers.add(6);
        circles.add("Siebeneck");         numbers.add(7);
        circles.add("Achteck");           numbers.add(8);
        circles.add("Neuneck");           numbers.add(9);
        circles.add("Zehneck");           numbers.add(10);
        circles.add("Zwölfeck");          numbers.add(12);
        circles.add("Fünfzehneck");       numbers.add(15);
        circles.add("Siebzehneck");       numbers.add(17);
        circles.add("neunzehneck");       numbers.add(19);
        circles.add("Zwanzigeck");        numbers.add(20);
        circles.add("Dreißigeck");        numbers.add(30);
        circles.add("Vierzigeck");        numbers.add(40);
        circles.add("Fünfzigeck");        numbers.add(50);
        circles.add("Neunundneunzigeck"); numbers.add(99);

        for (int index = 0; index < circles.size(); ++index) {
            String graphName = circles.get(index);
            int numberOfVertices = numbers.get(index);
            menu.add(createCircleMenuItem(graphName, numberOfVertices));
        }

        return menu;
    }

    private JMenuItem createCircleMenuItem(String graphName, int numberOfVertices) {
        return new MyMenuItem(graphName + " hinzufügen")
            //.mnemonic('m')
            //.accelerator(KeyEvent.VK_F, ctrlMask)
            .actionListener(e -> logic.insertCircle(numberOfVertices));
    }

    /** Erzeugt das Untermenü für die vollständigen Graphen. */
    private JMenu createTheInsertCompleteGraphsSubMenu() {
        JMenu menu = new JMenu("Vollständige Graphen hinzufügen");

        menu.setMnemonic('v');
        if (FileMenuCreation.OPAQUE_SUBMENUES) {
            menu.setOpaque(true);
        }

        List<Integer> numbers = CollectionsHelper.buildListFrom(
                3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, 17, 19, 20, 30, 40, 50, 99);

        for (int numberOfVertices : numbers) {
            menu.add(createCompleteGraphMenuItem(numberOfVertices));
        }

        return menu;
    }

    private JMenuItem createCompleteGraphMenuItem(int numberOfVertices) {
        return new MyMenuItem("K" + Integer.toString(numberOfVertices) + " hinzufügen")
            //.mnemonic('m')
            //.accelerator(KeyEvent.VK_F, ctrlMask)
            .actionListener(e -> logic.insertCompleteGraph(numberOfVertices));
    }

    private JMenuItem createInsertCenterMenuItem() {
        return new MyMenuItem("Mittelpunkt einfügen")
            .mnemonic('m')
            //.accelerator(KeyEvent.VK_F, ctrlMask)
            .actionListener(e -> logic.insertCenter());
    }

    private JMenuItem createMoveGraphLeftMenuItem() {
        return new MyMenuItem("Graphen nach links bewegen")
            //.mnemonic('k')
            //.accelerator(KeyEvent.VK_F, ctrlMask)
            .actionListener(e -> logic.moveLeft());
    }

    private JMenuItem createMoveGraphRightMenuItem() {
        return new MyMenuItem("Graphen nach rechts bewegen")
            //.mnemonic('k')
            //.accelerator(KeyEvent.VK_F, ctrlMask)
            .actionListener(e -> logic.moveRight());
    }

    private JMenuItem createMoveGraphUpMenuItem() {
        return new MyMenuItem("Graphen nach oben bewegen")
            //.mnemonic('k')
            //.accelerator(KeyEvent.VK_F, ctrlMask)
            .actionListener(e -> logic.moveUp());
    }

    private JMenuItem createMoveGraphDownMenuItem() {
        return new MyMenuItem("Graphen nach unten bewegen")
            //.mnemonic('k')
            //.accelerator(KeyEvent.VK_F, ctrlMask)
            .actionListener(e -> logic.moveDown());
    }

    private JMenuItem createSetAllVertexColorsToDefaultMenuItem() {
        return new MyMenuItem("Alle Eckenfarben auf Default setzen")
            //.mnemonic('k')
            //.accelerator(KeyEvent.VK_F, ctrlMask)
            .actionListener(e -> logic.setAllVertexColorsToDefault());
    }

    private JMenuItem createSetAllVertexRadiusToDefaultMenuItem() {
        return new MyMenuItem("Alle Eckenradien auf Default setzen")
            //.mnemonic('k')
            //.accelerator(KeyEvent.VK_F, ctrlMask)
            .actionListener(e -> logic.setAllVertexRadiusToDefault());
    }

    private JMenuItem createSetAllEdgeColorsToDefaultMenuItem() {
        return new MyMenuItem("Alle Kantenfarben auf Default setzen")
            //.mnemonic('k')
            //.accelerator(KeyEvent.VK_F, ctrlMask)
            .actionListener(e -> logic.setAllEdgeColorsToDefault());
    }

    private JMenuItem createSetAllEdgeWidthsToDefaultMenuItem() {
        return new MyMenuItem("Alle Kantendicken auf Default setzen")
            //.mnemonic('k')
            //.accelerator(KeyEvent.VK_F, ctrlMask)
            .actionListener(e -> logic.setAllEdgeWidthsToDefault());
    }

}
