package de.duehl.math.geometry;

/*
 * Copyright 2016 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

/**
 * Diese Klasse steht für einen zweidimensionalen Punkt.
 *
 * @version 1.01     2012-10-30
 * @author Christian Dühl
 */

public class Point {

    /** x-Koordinate des Punktes. */
    private double x;

    /** y-Koordinate des Punktes. */
    private double y;

    /** Default-Konstruktor. Erzeugt den Nullpunkt. */
    public Point() {
        this(0.0, 0.0);
    }

    /**
     * Konstruktor.
     *
     * @param x
     *            x-Koordinate des Punktes.
     * @param y
     *            y-Koordinate des Punktes.
     */
    public Point(double x, double y) {
        this.x = x;
        this.y = y;
    }

    /**
     * Copy-Konstruktor, kopiert die Koordinaten des gegebenen Punktes.
     *
     * @param that
     *            Punkt, von dem die Koordinaten kopiert werden sollen.
     */
    public Point(Point that) {
        this.x = that.x;
        this.y = that.y;
    }

    /**
     * Konstruktor, der die Koordinaten aus einem AWT-Punkt entnimmt.
     *
     * @param point
     *            AWT-Punkt.
     */
    public Point(java.awt.Point point) {
        x = point.getX();
        y = point.getY();
    }

    /** Getter für die x-Koordinate. */
    public double getX() {
        return x;
    }

    /** Setter für die x-Koordinate. */
    public void setX(double x) {
        this.x = x;
    }

    /** Getter für die y-Koordinate. */
    public double getY() {
        return y;
    }

    /** Setter für die y-Koordinate. */
    public void setY(double y) {
        this.y = y;
    }

    /** Getter für die x-Koordinate als Ganzzahlwert. */
    public int getXAsInteger() {
        return (int) Math.round(x);
    }

    /** Getter für die y-Koordinate als Ganzzahlwert. */
    public int getYAsInteger() {
        return (int) Math.round(y);
    }

    /**
     * Bestimmt die Entfernung zu einem anderen Punkt.
     *
     * @param that
     *            anderer Punkt
     * @return Entfernung zwischen diesem und dem anderen Punkt.
     */
    public double calculateDistanceTo(Point that) {
        Geometry g = new Geometry();
        return g.calculateDistanceBetweenPointAndPoint(this, that);
    }

    /**
     * Bestimmt die Entfernung zu einer Linie.
     *
     * @param line
     *            Linie
     * @return Entfernung zwischen dem Punkt und der Linie.
     */
    public double calculateDistanceTo(Line line) {
        Geometry g = new Geometry();
        return g.calculateDistanceBetweenPointAndLine(this, line);
    }

    /**
     * Test auf Gleichheit der Koordinaten von diesem und einem anderen Punkt
     * unter Berücksichtigung der Genauigkeit epsilon.
     *
     * @param that
     *            anderer Punkt
     * @param epsilon
     *            Genauigkeit
     * @return Wahrheitswert (true: Punkte liegen unter Berücksichtigung der
     *         Genauigkeit an gleichen Koordinaten)
     */
    public boolean equals(Point that, double epsilon) {
        double distance = calculateDistanceTo(that);
        return distance < epsilon;
    }


    /* --- generierte Methoden: --- */



    @Override
    public String toString() {
        return "Point [x=" + x + ", y=" + y + "]";
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        long temp;
        temp = Double.doubleToLongBits(x);
        result = prime * result + (int) (temp ^ (temp >>> 32));
        temp = Double.doubleToLongBits(y);
        result = prime * result + (int) (temp ^ (temp >>> 32));
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        Point other = (Point) obj;
        double epsilon = 0.001;
        if (Math.abs(x - other.x) > epsilon) {
            return false;
        }
        if (Math.abs(y - other.y) > epsilon) {
            return false;
        }
        return true;
    }

}
