package de.duehl.math.geometry.discrete;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.util.ArrayList;
import java.util.List;

/**
 * Diese Klasse stellt eine Gerade einer endlichen projektiven Ebene dar.
 *
 * Angelehnt an das Spiel Dobble wird jede Gerade durch ein Symbol repräsentiert. Jeder
 * Punkt entspricht dann einer Spielkarte mit den acht Symbolen (Ordnung 7 + 1).
 *
 * @version 1.01     2018-01-02
 * @author Christian Dühl
 */

public class FiniteProjectiveLine {

    /** Symbol der Geraden. */
    private final Symbol symbol;

    /** Liste der mit dieser Geraden incidenten Punkte. */
    private final List<FiniteProjectivePoint> incidentPoints;

    /** Maximale Anzahl der Punkte auf dieser Geraden. */
    private final int maximalNumberOfPoints;

    /**
     * Konstruktor.
     *
     * @param symbol
     *            Symbol der Geraden.
     * @param order
     *            Ordnung der Endlichen Projektiven Ebene.
     */
    public FiniteProjectiveLine(Symbol symbol, int order) {
        this.symbol = symbol;

        maximalNumberOfPoints = FiniteProjectivePlane.numberOfPointsOnEachLine(order);
        incidentPoints = new ArrayList<>();
    }

    /** Getter für das Symbol der Geraden. */
    public Symbol getSymbol() {
        return symbol;
    }

    /** Gibt an, ob die Gerade durch den übergebenen Punkt läuft. */
    public boolean containesPoint(FiniteProjectivePoint point) {
        return incidentPoints.contains(point);
    }

    /**
     * Es wird geprüft, ob die Gerade durch jeden Punkt höchstens einmal läuft.
     */
    public boolean isValid() {
        for (int i = 0; i < incidentPoints.size() - 2; ++i) {
            FiniteProjectivePoint lineI = incidentPoints.get(i);
            for (int j = i + 1; j < incidentPoints.size() - 1; ++j) {
                FiniteProjectivePoint lineJ = incidentPoints.get(j);
                if (lineI.equals(lineJ)) {
                    return false;
                }
            }
        }

        return true;
    }

    /** Gibt an, ob die Gerade vollständig besetzt ist. */
    public boolean isMaximal() {
        return incidentPoints.size() == maximalNumberOfPoints;
    }

    /** Gibt an, ob die Gerade zulässig und vollständig ist. */
    public boolean isComplete() {
        return isValid() && isMaximal();
    }

    /** Getter für die Liste der mit dieser Geraden incidenten Punkte. */
    public List<FiniteProjectivePoint> getIncidentPoints() {
        return incidentPoints;
    }

    /** Fügt einen incidenten Punkt hinzu. */
    public void addIncidentPoint(FiniteProjectivePoint point) {
        incidentPoints.add(point);
    }

    /** Entfernt einen incidenten Punkt. */
    public void removeIncidentPoint(FiniteProjectivePoint point) {
        incidentPoints.remove(point);
    }

    /** Stringrepräsentation. */
    @Override
    public String toString() {
        return "FiniteProjectiveLine [symbol=" + symbol + ", Points: " + getIncidentPointNames()
                + "]";
    }

    private List<String> getIncidentPointNames() {
        List<String> names = new ArrayList<>();
        for (FiniteProjectivePoint point : incidentPoints) {
            names.add(point.getName());
        }
        return names;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((symbol == null) ? 0 : symbol.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        FiniteProjectiveLine other = (FiniteProjectiveLine) obj;
        if (symbol == null) {
            if (other.symbol != null) {
                return false;
            }
        }
        else if (!symbol.equals(other.symbol)) {
            return false;
        }
        return true;
    }

}
