package de.duehl.math.geometry;

/*
 * Copyright 2016 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import static org.junit.Assert.*;

import org.junit.Test;

import de.duehl.math.geometry.Geometry;
import de.duehl.math.geometry.Line;
import de.duehl.math.geometry.Point;

public class GeometryTest {

    @Test
    public void create1() {
        Geometry g = new Geometry();
        assertNotNull(g);
    }


    /* --------------------- Point to Point Tests --------------------- */


    @Test
    public void pointToPointDistance1() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 1.0);
        Point q = new Point(1.0, 1.0);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |
         *   |
         * 1 |    * (p und q)
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndPoint(p, q);
        double expected = 0.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToPointDistance2() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 1.0);
        Point q = new Point(2.0, 1.0);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |
         *   |
         * 1 |    p    q
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndPoint(p, q);
        double expected = 1.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToPointDistance3() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 1.0);
        Point q = new Point(1.0, 2.0);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |    q
         *   |
         * 1 |    p
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndPoint(p, q);
        double expected = 1.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToPointDistance4() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 1.0);
        Point q = new Point(2.0, 2.0);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |         q
         *   |
         * 1 |    p
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndPoint(p, q);
        double expected = Math.sqrt(2.0);
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToPointDistance5() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 2.0);
        Point q = new Point(2.0, 1.0);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |    p
         *   |
         * 1 |         q
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndPoint(p, q);
        double expected = Math.sqrt(2.0);
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToPointDistance6() {
        Geometry g = new Geometry();
        Point p = new Point(2.0, 1.0);
        Point q = new Point(1.0, 2.0);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |    q
         *   |
         * 1 |         p
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndPoint(p, q);
        double expected = Math.sqrt(2.0);
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }



    /* --------------------- Point to Line Tests --------------------- */


    @Test
    public void pointToLineDistanceOnHorizontalLine1() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 1.0);
        Point a = new Point(1.0, 1.0);
        Point b = new Point(3.0, 1.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |
         *   |  (a, p)
         * 1 |    *---------b
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 0.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceOnHorizontalLine2() {
        Geometry g = new Geometry();
        Point p = new Point(2.0, 1.0);
        Point a = new Point(1.0, 1.0);
        Point b = new Point(3.0, 1.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |
         *   |
         * 1 |    a----p----b
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 0.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceOnHorizontalLine3() {
        Geometry g = new Geometry();
        Point p = new Point(3.0, 1.0);
        Point a = new Point(1.0, 1.0);
        Point b = new Point(3.0, 1.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |
         *   |            (b, p)
         * 1 |    a---------*
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 0.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceOnHorizontalLine4() {
        Geometry g = new Geometry();
        Point p = new Point(2.1357, 1.0);
        Point a = new Point(1.0, 1.0);
        Point b = new Point(3.0, 1.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |
         *   |
         * 1 |    a-----p---b
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 0.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceLeftOfHorizontalLine1() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 1.0);
        Point a = new Point(2.0, 1.0);
        Point b = new Point(4.0, 1.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |
         *   |
         * 1 |    p    a---------b
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 1.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceLeftOfHorizontalLine2() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 1.0);
        Point a = new Point(3.0, 1.0);
        Point b = new Point(5.0, 1.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |
         *   |
         * 1 |    p         a---------b
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 2.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceRightOfHorizontalLine1() {
        Geometry g = new Geometry();
        Point p = new Point(4.0, 1.0);
        Point a = new Point(1.0, 1.0);
        Point b = new Point(3.0, 1.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |
         *   |
         * 1 |    a---------b    p
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 1.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceRightOfHorizontalLine2() {
        Geometry g = new Geometry();
        Point p = new Point(5.0, 1.0);
        Point a = new Point(1.0, 1.0);
        Point b = new Point(3.0, 1.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |
         *   |
         * 1 |    a---------b         p
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 2.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceAboveHorizontalLine0() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 2.0);
        Point a = new Point(2.0, 1.0);
        Point b = new Point(4.0, 1.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |    p
         *   |
         * 1 |         a---------b
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = Math.sqrt(2.0);
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceAboveHorizontalLine1() {
        Geometry g = new Geometry();
        Point p = new Point(2.0, 2.0);
        Point a = new Point(2.0, 1.0);
        Point b = new Point(4.0, 1.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |         p
         *   |
         * 1 |         a---------b
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 1.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceAboveHorizontalLine2() {
        Geometry g = new Geometry();
        Point p = new Point(3.0, 2.0);
        Point a = new Point(2.0, 1.0);
        Point b = new Point(4.0, 1.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |              p
         *   |
         * 1 |         a---------b
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 1.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceAboveHorizontalLine3() {
        Geometry g = new Geometry();
        Point p = new Point(4.0, 2.0);
        Point a = new Point(2.0, 1.0);
        Point b = new Point(4.0, 1.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |                   p
         *   |
         * 1 |         a---------b
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 1.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceAboveHorizontalLine4() {
        Geometry g = new Geometry();
        Point p = new Point(5.0, 2.0);
        Point a = new Point(2.0, 1.0);
        Point b = new Point(4.0, 1.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |                        p
         *   |
         * 1 |         a---------b
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = Math.sqrt(2.0);
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceBelowHorizontalLine0() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 1.0);
        Point a = new Point(2.0, 2.0);
        Point b = new Point(4.0, 2.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |         a---------b
         *   |
         * 1 |    p
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = Math.sqrt(2.0);
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceBelowHorizontalLine1() {
        Geometry g = new Geometry();
        Point p = new Point(2.0, 1.0);
        Point a = new Point(2.0, 2.0);
        Point b = new Point(4.0, 2.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |         a---------b
         *   |
         * 1 |         p
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 1.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceBelowHorizontalLine2() {
        Geometry g = new Geometry();
        Point p = new Point(3.0, 1.0);
        Point a = new Point(2.0, 2.0);
        Point b = new Point(4.0, 2.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |         a---------b
         *   |
         * 1 |              p
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 1.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceBelowHorizontalLine3() {
        Geometry g = new Geometry();
        Point p = new Point(4.0, 1.0);
        Point a = new Point(2.0, 2.0);
        Point b = new Point(4.0, 2.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |         a---------b
         *   |
         * 1 |                   p
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 1.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceBelowHorizontalLine4() {
        Geometry g = new Geometry();
        Point p = new Point(5.0, 1.0);
        Point a = new Point(2.0, 2.0);
        Point b = new Point(4.0, 2.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |
         *   |
         * 2 |         a---------b
         *   |
         * 1 |                        p
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = Math.sqrt(2.0);
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceOnVerticalLine1() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 1.0);
        Point a = new Point(1.0, 1.0);
        Point b = new Point(1.0, 3.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |    b
         *   |    |
         * 2 |    |
         *   |    |
         * 1 |    * (a, p)
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 0.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceOnVerticalLine2() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 2.0);
        Point a = new Point(1.0, 1.0);
        Point b = new Point(1.0, 3.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |    b
         *   |    |
         * 2 |    p
         *   |    |
         * 1 |    a
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 0.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceOnVerticalLine3() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 3.0);
        Point a = new Point(1.0, 1.0);
        Point b = new Point(1.0, 3.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |    * (b, p)
         *   |    |
         * 2 |    |
         *   |    |
         * 1 |    a
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 0.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceOnVerticalLine4() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 2.1357);
        Point a = new Point(1.0, 1.0);
        Point b = new Point(1.0, 3.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |    b
         *   |    p
         * 2 |    |
         *   |    |
         * 1 |    a
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 0.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceBelowOfVerticalLine1() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 1.0);
        Point a = new Point(1.0, 2.0);
        Point b = new Point(1.0, 4.0);
        Line l = new Line(a, b);
        /*
         * 4 |    b
         *   |    |
         * 3 |    |
         *   |    |
         * 2 |    a
         *   |
         * 1 |    p
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 1.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceBelowOfVerticalLine2() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 1.0);
        Point a = new Point(1.0, 3.0);
        Point b = new Point(1.0, 4.0);
        Line l = new Line(a, b);
        /*
         * 4 |    b
         *   |    |
         * 3 |    a
         *   |
         * 2 |
         *   |
         * 1 |    p
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 2.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceAboveVerticalLine1() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 4.0);
        Point a = new Point(1.0, 3.0);
        Point b = new Point(1.0, 1.0);
        Line l = new Line(a, b);
        /*
         * 4 |    p
         *   |
         * 3 |    a
         *   |    |
         * 2 |    |
         *   |    |
         * 1 |    b
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 1.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceAboveVerticalLine2() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 4.0);
        Point a = new Point(1.0, 1.0);
        Point b = new Point(1.0, 2.0);
        Line l = new Line(a, b);
        /*
         * 4 |    p
         *   |
         * 3 |
         *   |
         * 2 |    b
         *   |    |
         * 1 |    a
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 2.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceRightOfVerticalLine0() {
        Geometry g = new Geometry();
        Point p = new Point(2.0, 1.0);
        Point a = new Point(1.0, 2.0);
        Point b = new Point(1.0, 3.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |    b
         *   |    |
         * 2 |    a
         *   |
         * 1 |         p
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = Math.sqrt(2.0);
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceRightOfVerticalLine1() {
        Geometry g = new Geometry();
        Point p = new Point(2.0, 1.0);
        Point a = new Point(1.0, 1.0);
        Point b = new Point(1.0, 3.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |    b
         *   |    |
         * 2 |    |
         *   |    |
         * 1 |    a    p
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 1.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceRightOfVerticalLine2() {
        Geometry g = new Geometry();
        Point p = new Point(2.0, 2.0);
        Point a = new Point(1.0, 1.0);
        Point b = new Point(1.0, 3.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |    b
         *   |    |
         * 2 |    |    p
         *   |    |
         * 1 |    a
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 1.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceRightOfVerticalLine3() {
        Geometry g = new Geometry();
        Point p = new Point(2.0, 3.0);
        Point a = new Point(1.0, 1.0);
        Point b = new Point(1.0, 3.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |    b    p
         *   |    |
         * 2 |    |
         *   |    |
         * 1 |    a
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 1.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceRightOfVerticalLine4() {
        Geometry g = new Geometry();
        Point p = new Point(2.0, 3.0);
        Point a = new Point(1.0, 1.0);
        Point b = new Point(1.0, 2.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |         p
         *   |
         * 2 |    b
         *   |    |
         * 1 |    a
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = Math.sqrt(2.0);
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceLeftOfVerticalLine0() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 1.0);
        Point a = new Point(2.0, 2.0);
        Point b = new Point(2.0, 3.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |         b
         *   |         |
         * 2 |         a
         *   |
         * 1 |    p
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = Math.sqrt(2.0);
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceLeftOfVerticalLine1() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 1.0);
        Point a = new Point(2.0, 1.0);
        Point b = new Point(2.0, 3.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |         b
         *   |         |
         * 2 |         |
         *   |         |
         * 1 |    p    a
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 1.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceLeftOfVerticalLine2() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 2.0);
        Point a = new Point(2.0, 1.0);
        Point b = new Point(2.0, 3.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |         b
         *   |         |
         * 2 |    p    |
         *   |         |
         * 1 |         a
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 1.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceLeftOfVerticalLine3() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 3.0);
        Point a = new Point(2.0, 1.0);
        Point b = new Point(2.0, 3.0);
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |    p    b
         *   |         |
         * 2 |         |
         *   |         |
         * 1 |         a
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = 1.0;
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceLeftOfVerticalLine4() {
        Geometry g = new Geometry();
        Point p = new Point(1.0, 4.0);
        Point a = new Point(2.0, 1.0);
        Point b = new Point(2.0, 3.0);
        Line l = new Line(a, b);
        /*
         * 4 |    p
         *   |
         * 3 |         b
         *   |         |
         * 2 |         |
         *   |         |
         * 1 |         a
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p, l);
        double expected = Math.sqrt(2.0);
        double epsilon = 0.001;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceWildTest1() {
        Geometry g = new Geometry();
        Point p1 = new Point(4.775, 2.72 );
        Point a  = new Point(0.6  , 2.3  );
        Point b  = new Point(4.62 , 0.82 );
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |                       p1
         *   |  a
         * 2 |
         *   |
         * 1 |                      b
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p1, l);
        double expected = 1.83;
        double epsilon = 0.075;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceWildTest2() {
        Geometry g = new Geometry();
        Point p2 = new Point(1.25 , 1.2  );
        Point a  = new Point(0.6  , 2.3  );
        Point b  = new Point(4.62 , 0.82 );
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |
         *   |  a
         * 2 |
         *   |     p2
         * 1 |                      b
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p2, l);
        double expected = 0.8;
        double epsilon = 0.01;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceWildTest3() {
        Geometry g = new Geometry();
        Point p3 = new Point(1.55 , 4.15 );
        Point a  = new Point(0.6  , 2.3  );
        Point b  = new Point(4.62 , 0.82 );
        Line l = new Line(a, b);
        /*
         * 4 |          p3
         *   |
         * 3 |
         *   |  a
         * 2 |
         *   |
         * 1 |                      b
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p3, l);
        double expected = 2.05;
        double epsilon = 0.05;
        assertEquals(expected, actual, epsilon);
    }

    @Test
    public void pointToLineDistanceWildTest4() {
        Geometry g = new Geometry();
        Point p4 = new Point(3.2  , 3.0  );
        Point a  = new Point(0.6  , 2.3  );
        Point b  = new Point(4.62 , 0.82 );
        Line l = new Line(a, b);
        /*
         * 4 |
         *   |
         * 3 |               p4
         *   |  a
         * 2 |
         *   |
         * 1 |                      b
         *   |
         * 0 +---------------------------
         *   0    1    2    3    4    5
         */
        double actual = g.calculateDistanceBetweenPointAndLine(p4, l);
        double expected = 1.55;
        double epsilon = 0.05;
        assertEquals(expected, actual, epsilon);
    }

}
