package de.duehl.swing.ui.elements;

/*
 * Copyright 2017 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.awt.Dimension;
import java.awt.Rectangle;

import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;

import de.duehl.swing.ui.GuiTools;

/**
 * Diese Klasse enthält einen Editor mit Scrollbalken.
 * Standardmäßig kann man den Text nicht editieren, dazu muss setEditable(true) aufgerufen werden.
 *
 * @version 1.01     2017-07-11
 * @author Christian Dühl
 */

public class EditorWithScrollBars {

    /** EditorPane. */
    private final JEditorPane editor;

    /** An der EditorPane befestigte Scrollbalken. */
    private final JScrollPane scroll;

    /**
     * Konstruktor.
     *
     * @param editor
     *            EditorPane.
     * @param scroll
     *            An der EditorPane befestigte Scrollbalken.
     */
    private EditorWithScrollBars(JEditorPane editor, JScrollPane scroll) {
        this.editor = editor;
        this.scroll = scroll;
    }

    /** Getter für die EditorPane. */
    public JEditorPane getEditor() {
        return editor;
    }

    /** Getter für die an der EditorPane befestigte Scrollbalken. */
    public JScrollPane getScroll() {
        return scroll;
    }

    /**
     * Fügt den Editor mit Scrollbalken einem Panel hinzu.
     *
     * @param panel
     *            Panel, dem der Editor mit Scrollbalken hinzugefügt werden soll.
     */
    public void bindOnPanel(JPanel panel) {
        panel.add(scroll);
    }

    /**
     * Fügt den Editor mit Scrollbalken einem Panel hinzu.
     *
     * @param panel
     *            Panel, dem der Editor mit Scrollbalken hinzugefügt werden soll.
     * @param constraints
     *            Angaben zum Layout für den Editor im Panel.
     */
    public void bindOnPanel(JPanel panel, String constraints) {
        panel.add(scroll, constraints);
    }

    /** Setzt die Scrollleiste auf den Minimalwert. */
    public void scrollScrollbarToMinimum() {
        //GuiTools.setScrollPaneToMinimum(scroll);
        GuiTools.scrollScrollbarToMinimum(scroll); // verzögerter Aufruf
    }

    /** Setzt die Scrollleiste auf den Maximalwert. */
    public void scrollScrollbarToMaximum() {
        //GuiTools.setScrollPaneToMaximum(scroll);
        GuiTools.scrollScrollbarToMaximum(scroll); // verzögerter Aufruf
    }

    /** Zeigt in der Scrollleiste den vorherigen Abschnitt an (etwa für PAGE-UP). */
    public void setScrollPaneToPreviousSection() {
        GuiTools.setScrollPaneToPreviousSection(scroll);
    }

    /** Zeigt in der Scrollleiste den nächsten Abschnitt an (etwa für PAGE-DOWN). */
    public void setScrollPaneToNextSection() {
        GuiTools.setScrollPaneToNextSection(scroll);
    }

    /**
     * Setzt den Text des Editors mit Scrollbalken.
     *
     * @param text
     *            Zu setzender Text.
     */
    public void setText(String text) {
        editor.setText(text);
    }

    /**
     * Returns the text contained in this TextComponent in terms of the content type of this
     * editor. If an exception is thrown while attempting to retrieve the text, null will be
     * returned. This is implemented to call JTextComponent.write with a StringWriter.
     */
    public String getText() {
        return editor.getText();
    }

    /**
     * Setzt den Text des Editors mit Scrollbalken und scrollt hinterher den Sichtbaren Bereich des
     * Textes zum Anfang des Textes.
     *
     * @param text
     *            Zu setzender Text.
     */
    public void setTextAndScrollToTop(String text) {
        setText(text);
        SwingUtilities.invokeLater(() -> scrollScrollbarToMinimum());
        /*
         * Funktioniert leider nur mit invokeLater... Vermutlich kommt es sonst vor dem setText() ?
         */
    }

    /**
     * Setzt die Position der Eingabemarke im Editor.
     *
     * @param position
     *            Zu setzende Position der Eingabemarke (0 bis Länge des Textes).
     */
    public void setCaretPosition(int position) {
        editor.setCaretPosition(position);
    }

    /** Zeichnet den Editor neu. */
    public void repaint() {
        editor.repaint();
        editor.validate();
        scroll.validate();
    }

    /**
     * Setzt die bevorzugte Größe des Editors.
     *
     * @param width
     *            Gewünschte Breite.
     * @param height
     *            Gewünschte Höhe.
     */
    public void setPreferredSize(int width, int height) {
        scroll.setPreferredSize(new Dimension(width, height));
    }

    /** Erzeugt ein einen Editor mit Scrollbalken. */
    public static EditorWithScrollBars createForHtml() {
        return create("text/html");
    }

    /** Erzeugt ein einen Editor mit Scrollbalken. */
    public static EditorWithScrollBars createForPlainText() {
        return create("text/plain");
    }

    /**
     * Erzeugt ein einen Editor mit Scrollbalken im gegebenen Content-Type.
     *
     * @param contentType
     *            Zu verwendender Content-Type (z.B. "text/html" oder "text/plain").
     */
    private static EditorWithScrollBars create(String contentType) {
        JEditorPane editor = new JEditorPane();
        editor.setContentType(contentType);
        editor.setEditable(false);
        //editor.setTransferHandler(new HtmlCopyPasteTransferHandler());

        JScrollPane scroll = new JScrollPane(editor);

        return new EditorWithScrollBars(editor, scroll);
    }

    /**
     * Turns on or off automatic drag handling. In order to enable automatic drag handling, this
     * property should be set to {@code true}, and the component's {@code TransferHandler} needs to
     * be {@code non-null}. The default value of the {@code dragEnabled} property is {@code false}.
     * <p>
     * The job of honoring this property, and recognizing a user drag gesture, lies with the look
     * and feel implementation, and in particular, the component's {@code TextUI}. When automatic
     * drag handling is enabled, most look and feels (including those that subclass
     * {@code BasicLookAndFeel}) begin a drag and drop operation whenever the user presses the
     * mouse button over a selection and then moves the mouse a few pixels. Setting this property
     * to {@code true} can therefore have a subtle effect on how selections behave.
     * <p>
     * If a look and feel is used that ignores this property, you can still begin a drag and drop
     * operation by calling {@code exportAsDrag} on the component's {@code TransferHandler}.
     *
     * @param enable
     *            whether or not to enable automatic drag handling
     */
    public void setDragEnabled(boolean enable) {
        editor.setDragEnabled(enable);
    }

    /**
     * Returns the position of the text insertion caret for the text component.
     *
     * @return the position of the text insertion caret for the text component &ge; 0
     */
    public int getCaretPosition() {
        return editor.getCaretPosition();
    }

    /**
     * Returns the <code>Component</code>'s "visible rectangle" - the intersection of this
     * component's visible rectangle, <code>new Rectangle(0, 0, getWidth(), getHeight())</code>,
     * and all of its ancestors' visible rectangles.
     *
     * @return the visible rectangle
     */
    public Rectangle getVisibleRect() {
        return editor.getVisibleRect();
    }

    /**
     * Forwards the <code>scrollRectToVisible()</code> message to the <code>JComponent</code>'s
     * parent. Components that can service the request, such as <code>JViewport</code>, override
     * this method and perform the scrolling.
     *
     * @param rectangle
     *            the visible <code>Rectangle</code>
     * @see JViewport
     */
    public void scrollRectToVisible(Rectangle rectangle) {
        editor.scrollRectToVisible(rectangle);
    }

    /**
     * Returns the selected text's start position. Return 0 for an empty document, or the value of
     * dot if no selection.
     *
     * @return the start position &ge; 0
     */
    public int getSelectionStart() {
        return editor.getSelectionStart();
    }

    /**
     * Returns the selected text's end position. Return 0 if the document is empty, or the value of
     * dot if there is no selection.
     *
     * @return the end position &ge; 0
     */
    public int getSelectionEnd() {
        return editor.getSelectionEnd();
    }

    /**
     * Sets the selection start to the specified position. The new starting point is constrained to
     * be before or at the current selection end. <p>
     *
     * This is available for backward compatibility to code that called this method on
     * <code>java.awt.TextComponent</code>. This is implemented to forward to the
     * <code>Caret</code> implementation which is where the actual selection is maintained.
     *
     * @param start
     *            the start position of the text &ge; 0
     */
    public void setSelectionStart(int start) {
        editor.setSelectionStart(start);
    }

    /**
     * Sets the selection end to the specified position. The new end point is constrained to be at
     * or after the current selection start. <p>
     *
     * This is available for backward compatibility to code that called this method on
     * <code>java.awt.TextComponent</code>. This is implemented to forward to the
     * <code>Caret</code> implementation which is where the actual selection is maintained.
     *
     * @param end
     *            the end position of the text &ge; 0
     */
    public void setSelectionEnd(int end) {
        editor.setSelectionEnd(end);
    }

    /**
     * Returns the selected text contained in this TextComponent. If the selection is null or the
     * document empty, returns null.
     */
    public String getSelectedText() {
        return editor.getSelectedText();
    }

    /**
     * Sets the specified boolean to indicate whether or not this TextComponent should be editable.
     * A PropertyChange event ("editable") is fired when the state is changed.
     */
    public void setEditable(boolean editable) {
        editor.setEditable(editable);
    }

}
